from __future__ import annotations
import datetime
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union
from uuid import UUID

if TYPE_CHECKING:
    from .budgets_get_response_results_budget_code import BudgetsGetResponse_results_budgetCode
    from .budgets_get_response_results_code_segment_values import BudgetsGetResponse_results_codeSegmentValues
    from .budgets_get_response_results_scope import BudgetsGetResponse_results_scope

@dataclass
class BudgetsGetResponse_results(Parsable):
    # The total amount of actual cost of the budget.
    actual_cost: Optional[float] = None
    # Not relevant
    actual_end_date: Optional[str] = None
    # The total of the actual quantity of labor, material,... for the budget.
    actual_quantity: Optional[float] = None
    # Not relevant
    actual_start_date: Optional[str] = None
    # Average unit price for the actual cost of the budget.
    actual_unit_price: Optional[float] = None
    # The total amount of changes committed to the supplier.
    approved_change_orders: Optional[float] = None
    # The total amount of in-scope changes committed to the supplier.
    approved_in_scope_change_orders: Optional[float] = None
    # The total amount of changes approved by the owner
    approved_owner_changes: Optional[float] = None
    # Budget code template and budget segment codes used in the budget. Not applicable for sub-budget.
    budget_code: Optional[BudgetsGetResponse_results_budgetCode] = None
    # Unique code compliant with the budget code template defined by the project admin.Max length: 255
    code: Optional[str] = None
    # Not relevant
    code_segment_values: Optional[BudgetsGetResponse_results_codeSegmentValues] = None
    # Not relevant
    contract_ids: Optional[list[str]] = None
    # Not relevant
    cost_unit_price: Optional[float] = None
    # The date and time that the item was created, in ISO 8601 format.
    created_at: Optional[datetime.datetime] = None
    # The detail description of the budget.Max length: 2048
    description: Optional[str] = None
    # Not relevant
    duration_days: Optional[float] = None
    # The identifier assigned to an item in its original external ERP system. Use this ID to track and look up data within the integrated system. Note that this value comes from the item's ID in the external system.Max length: 255
    external_id: Optional[str] = None
    # A message generated by the external ERP system that explains the sync status of the integration. For example, common values include ``success`` or ``fail`` to indicate the result of the integration operation.Max length: 255
    external_message: Optional[str] = None
    # The name of the external ERP system integrated with Cost Management. Use this name to identify and search for data within the integrated system.Max length: 255
    external_system: Optional[str] = None
    # The total amount of the forecast cost to complete, equals to ``forecastFinalCost`` - ``actualCost``.
    forecast_cost_complete: Optional[float] = None
    # The total amount of the project cost including forecast adjustments, equals to ``projectedCost`` + ``adjustmentsTotal``.
    forecast_final_cost: Optional[float] = None
    # The total amount of the forecast variance, equals to ``projectedBudget`` - ``forecastFinalCost``.
    forecast_variance: Optional[float] = None
    # Unique identifier of the budget.
    id: Optional[UUID] = None
    # The input quantity planned for the budget.
    input_quantity: Optional[float] = None
    # The state of the item during the integration with the external ERP system (such as SignNow). An item can be a ``budget``, ``contract``, ``main contract``, ``main contract item``, ``cost item``, ``expense``, ``expense item``, ``change order``, or ``schedule of value``. For more details, see `Integrate with External System </en/docs/acc/v1/tutorials/cost/integrate-with-external-system/>`_ tutorial.Possible values:``locked``: the item is currently locked within the ERP system, preventing modifications until unlocked. To unlock and modify the item, use the relevant PATCH endpoint to set ``integrationState`` to ``null``. For example, for a budget, call `PATCH budgets <en/docs/bim360/v1/reference/http/cost-budgets-budgetId-PATCH/>`_. For a contract, call `PATCH contracts <en/docs/bim360/v1/reference/http/cost-contracts-contractId-PATCH/>`_. For more details, see the `Help documentation <https://help.autodesk.com/view/BUILD/ENU/?guid=Integrated_and_Locked>`_.``integrated``: the item has been successfully added to the ERP system.``failed``: the item encountered an error during the integration process and was not successfully added to the ERP system. For example, if a user tries to integrate ``contracts`` from an ERP system and the updates fail, the ``integrationState`` can be set to ``failed``. Retry the sync process or analyze the issue if it continues to fail.``null``: The item has not been integrated with the ERP system. This is default value.For more information regarding integrations within the Cost Management system, see `Integrations in Cost Management <https://help.autodesk.com/view/BUILD/ENU/?guid=Cost_Integrations>`_.
    integration_state: Optional[str] = None
    # The date and time that the item's integration status was last changed.
    integration_state_changed_at: Optional[str] = None
    # The user who last changed the integration status. This is the ID of a user managed by the BIM 360/ACC Admin.
    integration_state_changed_by: Optional[str] = None
    # Amount of internal transfers between budgets
    internal_adjustment: Optional[float] = None
    # Not relevant
    is_markup: Optional[bool] = None
    # The date and time when the item was last synchronized with the external ERP system. This value is updated by the external system and is in ISO 8601 format.
    last_sync_time: Optional[datetime.datetime] = None
    # A list of the IDs of the project locations where this item applies, along with the node paths of these locations in the project's locations tree.For more information, see the Locations `Help documentation <https://aps.autodesk.com/en/docs/bim360/v1/reference/http/locations-nodes-GET/>`_ help.
    location_paths: Optional[list[str]] = None
    # A list of the IDs of the project locations where this item applies.For more information, see the Locations `Help documentation <https://aps.autodesk.com/en/docs/bim360/v1/reference/http/locations-nodes-GET/>`_ help.
    locations: Optional[list[str]] = None
    # Not relevant
    main_contract_amount_source: Optional[str] = None
    # Not relevant
    main_contract_id: Optional[UUID] = None
    # Not relevant
    milestone_id: Optional[str] = None
    # The name of the budget.Max length: 1024
    name: Optional[str] = None
    # Original amount of the budget, equals to ``quantity`` * ``unitPrice``
    original_amount: Optional[float] = None
    # The original amount committed in the supplier contract.
    original_commitment: Optional[float] = None
    # ID of a parent budget, ``null`` for root budgets.
    parent_id: Optional[str] = None
    # The total amount of changes not committed to the supplier.
    pending_change_orders: Optional[float] = None
    # The total amount of changes waiting for the approval by the owner
    pending_owner_changes: Optional[float] = None
    # Not relevant
    planned_end_date: Optional[str] = None
    # Not relevant
    planned_start_date: Optional[str] = None
    # The total amount of the project budget including pending changes, equals to ``revised`` + ``pendingOwnerChanges``.
    projected_budget: Optional[float] = None
    # The total amount of the project cost. For a budget, it is equal to ``originalCommitment`` + ``approvedChangeOrders`` + ``pendingChangeOrders`` + ``reserves``. For a contract , it is equal to ``awarded`` + ``approvedChangeOrders`` + ``pendingChangeOrders`` + ``reserves``.
    projected_cost: Optional[float] = None
    # The quantity of labor, material,... planned for the budget.
    quantity: Optional[float] = None
    # The ratio of the quantity the resources used on the budget.
    ratio: Optional[float] = None
    # The total amount of changes under estimation.
    reserves: Optional[float] = None
    # The total amount of the approved budget from the owner, equals to ``originalAmount`` + ``internalAdjustment`` + ``approvedOwnerChanges``.
    revised: Optional[float] = None
    # The scope of the budget. Possible values are ``budgetOnly`` and ``budgetAndCost``.
    scope: Optional[BudgetsGetResponse_results_scope] = None
    # The sub-budget in the root budget.Max length: 255
    sub_items: Optional[list[str]] = None
    # The amount that has been approved by owner but not committed to the supplier. This is calculated as ``approvedOwnerChanges`` - (``approvedChangeOrders`` - ``approvedInScopeChangeOrders``).
    uncommitted: Optional[float] = None
    # Unit of measures used in the budget.
    unit: Optional[str] = None
    # Unit price of a budget.
    unit_price: Optional[float] = None
    # The date and time that the item was last updated, in ISO 8601 format.
    updated_at: Optional[datetime.datetime] = None
    # The total amount of the variance of a budget, equals to ``projectedBudget`` - ``projectedCost``.
    variance_total: Optional[float] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> BudgetsGetResponse_results:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: BudgetsGetResponse_results
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return BudgetsGetResponse_results()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        from .budgets_get_response_results_budget_code import BudgetsGetResponse_results_budgetCode
        from .budgets_get_response_results_code_segment_values import BudgetsGetResponse_results_codeSegmentValues
        from .budgets_get_response_results_scope import BudgetsGetResponse_results_scope

        from .budgets_get_response_results_budget_code import BudgetsGetResponse_results_budgetCode
        from .budgets_get_response_results_code_segment_values import BudgetsGetResponse_results_codeSegmentValues
        from .budgets_get_response_results_scope import BudgetsGetResponse_results_scope

        fields: dict[str, Callable[[Any], None]] = {
            "actualCost": lambda n : setattr(self, 'actual_cost', n.get_float_value()),
            "actualEndDate": lambda n : setattr(self, 'actual_end_date', n.get_str_value()),
            "actualQuantity": lambda n : setattr(self, 'actual_quantity', n.get_float_value()),
            "actualStartDate": lambda n : setattr(self, 'actual_start_date', n.get_str_value()),
            "actualUnitPrice": lambda n : setattr(self, 'actual_unit_price', n.get_float_value()),
            "approvedChangeOrders": lambda n : setattr(self, 'approved_change_orders', n.get_float_value()),
            "approvedInScopeChangeOrders": lambda n : setattr(self, 'approved_in_scope_change_orders', n.get_float_value()),
            "approvedOwnerChanges": lambda n : setattr(self, 'approved_owner_changes', n.get_float_value()),
            "budgetCode": lambda n : setattr(self, 'budget_code', n.get_object_value(BudgetsGetResponse_results_budgetCode)),
            "code": lambda n : setattr(self, 'code', n.get_str_value()),
            "codeSegmentValues": lambda n : setattr(self, 'code_segment_values', n.get_object_value(BudgetsGetResponse_results_codeSegmentValues)),
            "contractIds": lambda n : setattr(self, 'contract_ids', n.get_collection_of_primitive_values(str)),
            "costUnitPrice": lambda n : setattr(self, 'cost_unit_price', n.get_float_value()),
            "createdAt": lambda n : setattr(self, 'created_at', n.get_datetime_value()),
            "description": lambda n : setattr(self, 'description', n.get_str_value()),
            "durationDays": lambda n : setattr(self, 'duration_days', n.get_float_value()),
            "externalId": lambda n : setattr(self, 'external_id', n.get_str_value()),
            "externalMessage": lambda n : setattr(self, 'external_message', n.get_str_value()),
            "externalSystem": lambda n : setattr(self, 'external_system', n.get_str_value()),
            "forecastCostComplete": lambda n : setattr(self, 'forecast_cost_complete', n.get_float_value()),
            "forecastFinalCost": lambda n : setattr(self, 'forecast_final_cost', n.get_float_value()),
            "forecastVariance": lambda n : setattr(self, 'forecast_variance', n.get_float_value()),
            "id": lambda n : setattr(self, 'id', n.get_uuid_value()),
            "inputQuantity": lambda n : setattr(self, 'input_quantity', n.get_float_value()),
            "integrationState": lambda n : setattr(self, 'integration_state', n.get_str_value()),
            "integrationStateChangedAt": lambda n : setattr(self, 'integration_state_changed_at', n.get_str_value()),
            "integrationStateChangedBy": lambda n : setattr(self, 'integration_state_changed_by', n.get_str_value()),
            "internalAdjustment": lambda n : setattr(self, 'internal_adjustment', n.get_float_value()),
            "isMarkup": lambda n : setattr(self, 'is_markup', n.get_bool_value()),
            "lastSyncTime": lambda n : setattr(self, 'last_sync_time', n.get_datetime_value()),
            "locationPaths": lambda n : setattr(self, 'location_paths', n.get_collection_of_primitive_values(str)),
            "locations": lambda n : setattr(self, 'locations', n.get_collection_of_primitive_values(str)),
            "mainContractAmountSource": lambda n : setattr(self, 'main_contract_amount_source', n.get_str_value()),
            "mainContractId": lambda n : setattr(self, 'main_contract_id', n.get_uuid_value()),
            "milestoneId": lambda n : setattr(self, 'milestone_id', n.get_str_value()),
            "name": lambda n : setattr(self, 'name', n.get_str_value()),
            "originalAmount": lambda n : setattr(self, 'original_amount', n.get_float_value()),
            "originalCommitment": lambda n : setattr(self, 'original_commitment', n.get_float_value()),
            "parentId": lambda n : setattr(self, 'parent_id', n.get_str_value()),
            "pendingChangeOrders": lambda n : setattr(self, 'pending_change_orders', n.get_float_value()),
            "pendingOwnerChanges": lambda n : setattr(self, 'pending_owner_changes', n.get_float_value()),
            "plannedEndDate": lambda n : setattr(self, 'planned_end_date', n.get_str_value()),
            "plannedStartDate": lambda n : setattr(self, 'planned_start_date', n.get_str_value()),
            "projectedBudget": lambda n : setattr(self, 'projected_budget', n.get_float_value()),
            "projectedCost": lambda n : setattr(self, 'projected_cost', n.get_float_value()),
            "quantity": lambda n : setattr(self, 'quantity', n.get_float_value()),
            "ratio": lambda n : setattr(self, 'ratio', n.get_float_value()),
            "reserves": lambda n : setattr(self, 'reserves', n.get_float_value()),
            "revised": lambda n : setattr(self, 'revised', n.get_float_value()),
            "scope": lambda n : setattr(self, 'scope', n.get_enum_value(BudgetsGetResponse_results_scope)),
            "subItems": lambda n : setattr(self, 'sub_items', n.get_collection_of_primitive_values(str)),
            "uncommitted": lambda n : setattr(self, 'uncommitted', n.get_float_value()),
            "unit": lambda n : setattr(self, 'unit', n.get_str_value()),
            "unitPrice": lambda n : setattr(self, 'unit_price', n.get_float_value()),
            "updatedAt": lambda n : setattr(self, 'updated_at', n.get_datetime_value()),
            "varianceTotal": lambda n : setattr(self, 'variance_total', n.get_float_value()),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_float_value("actualCost", self.actual_cost)
        writer.write_str_value("actualEndDate", self.actual_end_date)
        writer.write_float_value("actualQuantity", self.actual_quantity)
        writer.write_str_value("actualStartDate", self.actual_start_date)
        writer.write_float_value("actualUnitPrice", self.actual_unit_price)
        writer.write_float_value("approvedChangeOrders", self.approved_change_orders)
        writer.write_float_value("approvedInScopeChangeOrders", self.approved_in_scope_change_orders)
        writer.write_float_value("approvedOwnerChanges", self.approved_owner_changes)
        writer.write_object_value("budgetCode", self.budget_code)
        writer.write_str_value("code", self.code)
        writer.write_object_value("codeSegmentValues", self.code_segment_values)
        writer.write_collection_of_primitive_values("contractIds", self.contract_ids)
        writer.write_float_value("costUnitPrice", self.cost_unit_price)
        writer.write_datetime_value("createdAt", self.created_at)
        writer.write_str_value("description", self.description)
        writer.write_float_value("durationDays", self.duration_days)
        writer.write_str_value("externalId", self.external_id)
        writer.write_str_value("externalMessage", self.external_message)
        writer.write_str_value("externalSystem", self.external_system)
        writer.write_float_value("forecastCostComplete", self.forecast_cost_complete)
        writer.write_float_value("forecastFinalCost", self.forecast_final_cost)
        writer.write_float_value("forecastVariance", self.forecast_variance)
        writer.write_uuid_value("id", self.id)
        writer.write_float_value("inputQuantity", self.input_quantity)
        writer.write_str_value("integrationState", self.integration_state)
        writer.write_str_value("integrationStateChangedAt", self.integration_state_changed_at)
        writer.write_str_value("integrationStateChangedBy", self.integration_state_changed_by)
        writer.write_float_value("internalAdjustment", self.internal_adjustment)
        writer.write_bool_value("isMarkup", self.is_markup)
        writer.write_datetime_value("lastSyncTime", self.last_sync_time)
        writer.write_collection_of_primitive_values("locationPaths", self.location_paths)
        writer.write_collection_of_primitive_values("locations", self.locations)
        writer.write_str_value("mainContractAmountSource", self.main_contract_amount_source)
        writer.write_uuid_value("mainContractId", self.main_contract_id)
        writer.write_str_value("milestoneId", self.milestone_id)
        writer.write_str_value("name", self.name)
        writer.write_float_value("originalAmount", self.original_amount)
        writer.write_float_value("originalCommitment", self.original_commitment)
        writer.write_str_value("parentId", self.parent_id)
        writer.write_float_value("pendingChangeOrders", self.pending_change_orders)
        writer.write_float_value("pendingOwnerChanges", self.pending_owner_changes)
        writer.write_str_value("plannedEndDate", self.planned_end_date)
        writer.write_str_value("plannedStartDate", self.planned_start_date)
        writer.write_float_value("projectedBudget", self.projected_budget)
        writer.write_float_value("projectedCost", self.projected_cost)
        writer.write_float_value("quantity", self.quantity)
        writer.write_float_value("ratio", self.ratio)
        writer.write_float_value("reserves", self.reserves)
        writer.write_float_value("revised", self.revised)
        writer.write_enum_value("scope", self.scope)
        writer.write_collection_of_primitive_values("subItems", self.sub_items)
        writer.write_float_value("uncommitted", self.uncommitted)
        writer.write_str_value("unit", self.unit)
        writer.write_float_value("unitPrice", self.unit_price)
        writer.write_datetime_value("updatedAt", self.updated_at)
        writer.write_float_value("varianceTotal", self.variance_total)
    




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union
from uuid import UUID

if TYPE_CHECKING:
    from .metadata_get_response_admin_mappings_submittal_roles import MetadataGetResponse_adminMappingsSubmittalRoles
    from .metadata_get_response_attachment_categories import MetadataGetResponse_attachmentCategories
    from .metadata_get_response_attachment_types import MetadataGetResponse_attachmentTypes
    from .metadata_get_response_attachment_urn_types import MetadataGetResponse_attachmentUrnTypes
    from .metadata_get_response_custom_identifier_sequence_type import MetadataGetResponse_customIdentifierSequenceType
    from .metadata_get_response_default_values import MetadataGetResponse_defaultValues
    from .metadata_get_response_item_types import MetadataGetResponse_itemTypes
    from .metadata_get_response_responses import MetadataGetResponse_responses
    from .metadata_get_response_response_categories import MetadataGetResponse_responseCategories
    from .metadata_get_response_statuses import MetadataGetResponse_statuses
    from .metadata_get_response_submittal_roles import MetadataGetResponse_submittalRoles
    from .metadata_get_response_user_types import MetadataGetResponse_userTypes

@dataclass
class MetadataGetResponse(Parsable):
    # A list of admin mappings for submittal roles.
    admin_mappings_submittal_roles: Optional[list[MetadataGetResponse_adminMappingsSubmittalRoles]] = None
    # A list of attachment categories.
    attachment_categories: Optional[list[MetadataGetResponse_attachmentCategories]] = None
    # A list of attachment types.
    attachment_types: Optional[list[MetadataGetResponse_attachmentTypes]] = None
    # A list of attachment URN types.
    attachment_urn_types: Optional[list[MetadataGetResponse_attachmentUrnTypes]] = None
    # The custom numbering sequence type for the current project. Possible values: ``1`` (Global sequence), ``2`` (Spec sequence).
    custom_identifier_sequence_type: Optional[MetadataGetResponse_customIdentifierSequenceType] = None
    # An object containing the default values for various settings and configurations in the project.
    default_values: Optional[list[MetadataGetResponse_defaultValues]] = None
    # The unique identifier (UUID) for the metadata object.
    id: Optional[UUID] = None
    # ``true``: if there is at least one manager mapping in the project.``false``: if there are no manager mappings in the project.
    is_manager_mapping_defined: Optional[bool] = None
    # A list of submittal item types. This is the same as calling `GET item-types </en/docs/acc/v1/reference/http/submittals-item-types-GET/>`_
    item_types: Optional[list[MetadataGetResponse_itemTypes]] = None
    # ``true``: if there are no submittal items in the project.``false``: if there are submittal items in the project.
    no_items_in_project: Optional[bool] = None
    # ``true``: if there are no packages in the project.``false``: if there are packages in the project.
    no_packages_in_project: Optional[bool] = None
    # A list of categories for responses to submittals.
    response_categories: Optional[list[MetadataGetResponse_responseCategories]] = None
    # A list of responses.This is the same as calling `GET responses </en/docs/acc/v1/reference/http/submittals-responses-GET/>`_
    responses: Optional[list[MetadataGetResponse_responses]] = None
    # A list of statuses representing different stages of a submittal item.
    statuses: Optional[list[MetadataGetResponse_statuses]] = None
    # A list of submittal roles within the project.
    submittal_roles: Optional[list[MetadataGetResponse_submittalRoles]] = None
    # Types of users, such as individual users, companies, and roles.
    user_types: Optional[list[MetadataGetResponse_userTypes]] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> MetadataGetResponse:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: MetadataGetResponse
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return MetadataGetResponse()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        from .metadata_get_response_admin_mappings_submittal_roles import MetadataGetResponse_adminMappingsSubmittalRoles
        from .metadata_get_response_attachment_categories import MetadataGetResponse_attachmentCategories
        from .metadata_get_response_attachment_types import MetadataGetResponse_attachmentTypes
        from .metadata_get_response_attachment_urn_types import MetadataGetResponse_attachmentUrnTypes
        from .metadata_get_response_custom_identifier_sequence_type import MetadataGetResponse_customIdentifierSequenceType
        from .metadata_get_response_default_values import MetadataGetResponse_defaultValues
        from .metadata_get_response_item_types import MetadataGetResponse_itemTypes
        from .metadata_get_response_responses import MetadataGetResponse_responses
        from .metadata_get_response_response_categories import MetadataGetResponse_responseCategories
        from .metadata_get_response_statuses import MetadataGetResponse_statuses
        from .metadata_get_response_submittal_roles import MetadataGetResponse_submittalRoles
        from .metadata_get_response_user_types import MetadataGetResponse_userTypes

        from .metadata_get_response_admin_mappings_submittal_roles import MetadataGetResponse_adminMappingsSubmittalRoles
        from .metadata_get_response_attachment_categories import MetadataGetResponse_attachmentCategories
        from .metadata_get_response_attachment_types import MetadataGetResponse_attachmentTypes
        from .metadata_get_response_attachment_urn_types import MetadataGetResponse_attachmentUrnTypes
        from .metadata_get_response_custom_identifier_sequence_type import MetadataGetResponse_customIdentifierSequenceType
        from .metadata_get_response_default_values import MetadataGetResponse_defaultValues
        from .metadata_get_response_item_types import MetadataGetResponse_itemTypes
        from .metadata_get_response_responses import MetadataGetResponse_responses
        from .metadata_get_response_response_categories import MetadataGetResponse_responseCategories
        from .metadata_get_response_statuses import MetadataGetResponse_statuses
        from .metadata_get_response_submittal_roles import MetadataGetResponse_submittalRoles
        from .metadata_get_response_user_types import MetadataGetResponse_userTypes

        fields: dict[str, Callable[[Any], None]] = {
            "adminMappingsSubmittalRoles": lambda n : setattr(self, 'admin_mappings_submittal_roles', n.get_collection_of_object_values(MetadataGetResponse_adminMappingsSubmittalRoles)),
            "attachmentCategories": lambda n : setattr(self, 'attachment_categories', n.get_collection_of_object_values(MetadataGetResponse_attachmentCategories)),
            "attachmentTypes": lambda n : setattr(self, 'attachment_types', n.get_collection_of_object_values(MetadataGetResponse_attachmentTypes)),
            "attachmentUrnTypes": lambda n : setattr(self, 'attachment_urn_types', n.get_collection_of_object_values(MetadataGetResponse_attachmentUrnTypes)),
            "customIdentifierSequenceType": lambda n : setattr(self, 'custom_identifier_sequence_type', n.get_enum_value(MetadataGetResponse_customIdentifierSequenceType)),
            "defaultValues": lambda n : setattr(self, 'default_values', n.get_collection_of_object_values(MetadataGetResponse_defaultValues)),
            "id": lambda n : setattr(self, 'id', n.get_uuid_value()),
            "isManagerMappingDefined": lambda n : setattr(self, 'is_manager_mapping_defined', n.get_bool_value()),
            "itemTypes": lambda n : setattr(self, 'item_types', n.get_collection_of_object_values(MetadataGetResponse_itemTypes)),
            "noItemsInProject": lambda n : setattr(self, 'no_items_in_project', n.get_bool_value()),
            "noPackagesInProject": lambda n : setattr(self, 'no_packages_in_project', n.get_bool_value()),
            "responseCategories": lambda n : setattr(self, 'response_categories', n.get_collection_of_object_values(MetadataGetResponse_responseCategories)),
            "responses": lambda n : setattr(self, 'responses', n.get_collection_of_object_values(MetadataGetResponse_responses)),
            "statuses": lambda n : setattr(self, 'statuses', n.get_collection_of_object_values(MetadataGetResponse_statuses)),
            "submittalRoles": lambda n : setattr(self, 'submittal_roles', n.get_collection_of_object_values(MetadataGetResponse_submittalRoles)),
            "userTypes": lambda n : setattr(self, 'user_types', n.get_collection_of_object_values(MetadataGetResponse_userTypes)),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_collection_of_object_values("adminMappingsSubmittalRoles", self.admin_mappings_submittal_roles)
        writer.write_collection_of_object_values("attachmentCategories", self.attachment_categories)
        writer.write_collection_of_object_values("attachmentTypes", self.attachment_types)
        writer.write_collection_of_object_values("attachmentUrnTypes", self.attachment_urn_types)
        writer.write_enum_value("customIdentifierSequenceType", self.custom_identifier_sequence_type)
        writer.write_collection_of_object_values("defaultValues", self.default_values)
        writer.write_uuid_value("id", self.id)
        writer.write_bool_value("isManagerMappingDefined", self.is_manager_mapping_defined)
        writer.write_collection_of_object_values("itemTypes", self.item_types)
        writer.write_bool_value("noItemsInProject", self.no_items_in_project)
        writer.write_bool_value("noPackagesInProject", self.no_packages_in_project)
        writer.write_collection_of_object_values("responseCategories", self.response_categories)
        writer.write_collection_of_object_values("responses", self.responses)
        writer.write_collection_of_object_values("statuses", self.statuses)
        writer.write_collection_of_object_values("submittalRoles", self.submittal_roles)
        writer.write_collection_of_object_values("userTypes", self.user_types)
    




from enum import Enum

class TakeoffTypesGetResponse_results_propertyDefinitions_valueLocation(str, Enum):
    ValueLocation = "valueLocation",
    INSTANCE_WITH_TAKEOFF_TYPE_DEFAULT = "INSTANCE_WITH_TAKEOFF_TYPE_DEFAULT",
    TAKEOFF_TYPE = "TAKEOFF_TYPE",




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union
from uuid import UUID

@dataclass
class StatusStepSetsBatchGetPostResponse_results(Parsable):
    # The ID of the category to which the status set is assigned.
    category_id: Optional[str] = None
    # The ID of the category from which this status set was inherited - that is, the ancestor categoryto which the inherited status set was explicitly assigned. This field is only present when thisendpoint is set to return inherited status sets, and then only for inherited status sets.
    inherited_from_category_id: Optional[str] = None
    # The ID of the status set assigned to the category.
    status_step_set_id: Optional[UUID] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> StatusStepSetsBatchGetPostResponse_results:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: StatusStepSetsBatchGetPostResponse_results
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return StatusStepSetsBatchGetPostResponse_results()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        fields: dict[str, Callable[[Any], None]] = {
            "categoryId": lambda n : setattr(self, 'category_id', n.get_str_value()),
            "inheritedFromCategoryId": lambda n : setattr(self, 'inherited_from_category_id', n.get_str_value()),
            "statusStepSetId": lambda n : setattr(self, 'status_step_set_id', n.get_uuid_value()),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_str_value("categoryId", self.category_id)
        writer.write_str_value("inheritedFromCategoryId", self.inherited_from_category_id)
        writer.write_uuid_value("statusStepSetId", self.status_step_set_id)
    




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union

if TYPE_CHECKING:
    from .users_post_request_body_products_access import UsersPostRequestBody_products_access
    from .users_post_request_body_products_key import UsersPostRequestBody_products_key

@dataclass
class UsersPostRequestBody_products(Parsable):
    # The user's type of access to the product identified by ``key``. Possible values:- ``administrator``- ``member``- ``none``Note that when you're using a POST or PATCH endpoint to set this value, you must adhere to the following guidelines:- If you set a product's ``key`` to ``projectAdministration`` and you set ``access`` to ``none``, all other products should be set to ``member`` access for the user.- If you set a product's ``key`` to ``projectAdministration`` and you set ``access`` to ``administrator``, all other products should be set to ``administrator`` access for the user.- You cannot set a product's ``key`` to ``projectAdministration`` and set ``access`` to ``member``.
    access: Optional[UsersPostRequestBody_products_access] = None
    # A machine-readable identifier for the product (e.g., docs, build).Each product has a unique key used throughout the API for identification, filtering, and integration logic (e.g., in query parameters like ``filter[key]``).Possible values:ACC - ``autoSpecs``, ``build``, ``cost``, ``designCollaboration``, ``docs``, ``insight``, ``modelCoordination``, ``projectAdministration``, and ``takeoff``.BIM 360 - ``assets``, ``costManagement``, ``designCollaboration``, ``documentManagement``, ``field``, ``fieldManagement``, ``glue``, ``insight``, ``modelCoordination``, ``plan``, ``projectAdministration``, ``projectHome``, ``projectManagement``, and ``quantification``.Note that this endpoint returns only ACC products. Other endpoints, such as `GET projects </en/docs/acc/v1/reference/http/admin-accountsaccountidprojects-GET/>`_ and `GET projects/:projectId </en/docs/acc/v1/reference/http/admin-projects-projectId-GET/>`_, may return both ACC and BIM 360 projects. In those responses, product keys may include BIM 360 values.
    key: Optional[UsersPostRequestBody_products_key] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> UsersPostRequestBody_products:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: UsersPostRequestBody_products
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return UsersPostRequestBody_products()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        from .users_post_request_body_products_access import UsersPostRequestBody_products_access
        from .users_post_request_body_products_key import UsersPostRequestBody_products_key

        from .users_post_request_body_products_access import UsersPostRequestBody_products_access
        from .users_post_request_body_products_key import UsersPostRequestBody_products_key

        fields: dict[str, Callable[[Any], None]] = {
            "access": lambda n : setattr(self, 'access', n.get_enum_value(UsersPostRequestBody_products_access)),
            "key": lambda n : setattr(self, 'key', n.get_enum_value(UsersPostRequestBody_products_key)),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_enum_value("access", self.access)
        writer.write_enum_value("key", self.key)
    




from enum import Enum

class ItemsPostRequestBody_stateId(str, Enum):
    Draft = "draft",
    Sbc = "sbc",
    Mgr = "mgr",
    Subcontractor = "subcontractor",
    SubcontractorType = "subcontractorType",
    SubmitterDueDate = "submitterDueDate",
    Manager = "manager",




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union

@dataclass
class ViewsGetResponse_modelSetViewVersions_documentVersions_documentLineage(Parsable):
    """
    A document from a model set.
    """
    # Indicates whether the document lineage is aligned.
    is_aligned: Optional[bool] = None
    # The URN of the document lineage.
    lineage_urn: Optional[str] = None
    # The URN of the parent folder for the document lineage.
    parent_folder_urn: Optional[str] = None
    # The tip version URN for the document lineage.
    tip_version_urn: Optional[str] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> ViewsGetResponse_modelSetViewVersions_documentVersions_documentLineage:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: ViewsGetResponse_modelSetViewVersions_documentVersions_documentLineage
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return ViewsGetResponse_modelSetViewVersions_documentVersions_documentLineage()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        fields: dict[str, Callable[[Any], None]] = {
            "isAligned": lambda n : setattr(self, 'is_aligned', n.get_bool_value()),
            "lineageUrn": lambda n : setattr(self, 'lineage_urn', n.get_str_value()),
            "parentFolderUrn": lambda n : setattr(self, 'parent_folder_urn', n.get_str_value()),
            "tipVersionUrn": lambda n : setattr(self, 'tip_version_urn', n.get_str_value()),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_bool_value("isAligned", self.is_aligned)
        writer.write_str_value("lineageUrn", self.lineage_urn)
        writer.write_str_value("parentFolderUrn", self.parent_folder_urn)
        writer.write_str_value("tipVersionUrn", self.tip_version_urn)
    




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.base_request_builder import BaseRequestBuilder
from kiota_abstractions.base_request_configuration import RequestConfiguration
from kiota_abstractions.default_query_parameters import QueryParameters
from kiota_abstractions.get_path_parameters import get_path_parameters
from kiota_abstractions.method import Method
from kiota_abstractions.request_adapter import RequestAdapter
from kiota_abstractions.request_information import RequestInformation
from kiota_abstractions.request_option import RequestOption
from kiota_abstractions.serialization import Parsable, ParsableFactory
from typing import Any, Optional, TYPE_CHECKING, Union
from warnings import warn

if TYPE_CHECKING:
    from .item.packages_item_request_builder import PackagesItemRequestBuilder
    from .packages_get_response import PackagesGetResponse

class PackagesRequestBuilder(BaseRequestBuilder):
    """
    Builds and executes requests for operations under /construction/submittals/v2/projects/{projectId}/packages
    """
    def __init__(self,request_adapter: RequestAdapter, path_parameters: Union[str, dict[str, Any]]) -> None:
        """
        Instantiates a new PackagesRequestBuilder and sets the default values.
        param path_parameters: The raw url or the url-template parameters for the request.
        param request_adapter: The request adapter to use to execute the requests.
        Returns: None
        """
        super().__init__(request_adapter, "{+baseurl}/construction/submittals/v2/projects/{projectId}/packages{?filter%5Bidentifier%5D*,filter%5Bspec%26%2346%3Bidentifier%5D*,filter%5BspecId%5D*,filter%5Btitle%5D*,limit*,offset*,search*,sort*}", path_parameters)
    
    def by_id(self,id: str) -> PackagesItemRequestBuilder:
        """
        Gets an item from the Autodesk.ACC.construction.submittals.v2.projects.item.packages.item collection
        param id: The ID of the submittal item to retrieve revisions for. To obtain this ID, call `GET items </en/docs/acc/v1/reference/http/submittals-items-GET/>`_.
        Returns: PackagesItemRequestBuilder
        """
        if id is None:
            raise TypeError("id cannot be null.")
        from .item.packages_item_request_builder import PackagesItemRequestBuilder

        url_tpl_params = get_path_parameters(self.path_parameters)
        url_tpl_params["id"] = id
        return PackagesItemRequestBuilder(self.request_adapter, url_tpl_params)
    
    async def get(self,request_configuration: Optional[RequestConfiguration[PackagesRequestBuilderGetQueryParameters]] = None) -> Optional[PackagesGetResponse]:
        """
        Retrieve all the packages for the specified project. For information about packages, see the `Help documentation <https://help.autodesk.com/view/BUILD/ENU/?guid=Submittal_Packages>`_.
        param request_configuration: Configuration for the request such as headers, query parameters, and middleware options.
        Returns: Optional[PackagesGetResponse]
        """
        request_info = self.to_get_request_information(
            request_configuration
        )
        if not self.request_adapter:
            raise Exception("Http core is null") 
        from .packages_get_response import PackagesGetResponse

        return await self.request_adapter.send_async(request_info, PackagesGetResponse, None)
    
    def to_get_request_information(self,request_configuration: Optional[RequestConfiguration[PackagesRequestBuilderGetQueryParameters]] = None) -> RequestInformation:
        """
        Retrieve all the packages for the specified project. For information about packages, see the `Help documentation <https://help.autodesk.com/view/BUILD/ENU/?guid=Submittal_Packages>`_.
        param request_configuration: Configuration for the request such as headers, query parameters, and middleware options.
        Returns: RequestInformation
        """
        request_info = RequestInformation(Method.GET, self.url_template, self.path_parameters)
        request_info.configure(request_configuration)
        request_info.headers.try_add("Accept", "application/json")
        return request_info
    
    def with_url(self,raw_url: str) -> PackagesRequestBuilder:
        """
        Returns a request builder with the provided arbitrary URL. Using this method means any other path or query parameters are ignored.
        param raw_url: The raw URL to use for the request builder.
        Returns: PackagesRequestBuilder
        """
        if raw_url is None:
            raise TypeError("raw_url cannot be null.")
        return PackagesRequestBuilder(self.request_adapter, raw_url)
    
    @dataclass
    class PackagesRequestBuilderGetQueryParameters():
        """
        Retrieve all the packages for the specified project. For information about packages, see the `Help documentation <https://help.autodesk.com/view/BUILD/ENU/?guid=Submittal_Packages>`_.
        """
        def get_query_parameter(self,original_name: str) -> str:
            """
            Maps the query parameters names to their encoded names for the URI template parsing.
            param original_name: The original query parameter name in the class.
            Returns: str
            """
            if original_name is None:
                raise TypeError("original_name cannot be null.")
            if original_name == "filteridentifier":
                return "filter%5Bidentifier%5D"
            if original_name == "filterspec46identifier":
                return "filter%5Bspec%26%2346%3Bidentifier%5D"
            if original_name == "filterspec_id":
                return "filter%5BspecId%5D"
            if original_name == "filtertitle":
                return "filter%5Btitle%5D"
            if original_name == "limit":
                return "limit"
            if original_name == "offset":
                return "offset"
            if original_name == "search":
                return "search"
            if original_name == "sort":
                return "sort"
            return original_name
        
        # Filter packages with the specified package ID (the package ID in the UI). You can specify multiple values. Separate multiple values with commas. For example, ``filter[identifier]=2``.
        filteridentifier: Optional[str] = None

        # Filter packages with the associated specified spec section internal, globally unique ID (UUID). You can specify multiple values. Separate multiple values with commas. For example, ``filter[specId]=b4aa3864-5706-4a7b-b06c-a792e8b2df23``.
        filterspec_id: Optional[str] = None

        # Filter packages with the associated specified section ID (the spec section ID in the UI). You can specify multiple values. Separate multiple values with commas. For example, ``filter[identifier]=2``.
        filterspec46identifier: Optional[str] = None

        # Filter packages with the specified title. You can specify multiple values. Separate multiple values with commas. For example, ``filter[title]=Structural Steel``.
        filtertitle: Optional[str] = None

        # The maximum number of results per page. Possible values: ``1``- ``50``. Default value: ``20``. For example, to limit the response to two results per page, use ``limit=2``.
        limit: Optional[int] = None

        # The number of results to skip before starting to return data. For example, to skip the first 20 results, include ``offset=20`` in the query string. For more details, see the `JSON API Paging Help documentation <https://jsonapi.org/format/#fetching-pagination>`_.
        offset: Optional[int] = None

        # Search for packages by querying a specified string within specific fields (``identifier``, ``title``, ``spec.identifier``), and retrieve the associated packages that match the search criteria. This includes packages where the string matches part of a field. For example, ``search=1``.
        search: Optional[str] = None

        # Sort packages by specified fields. Separate multiple values with commas. To sort in descending or ascending order, add ``desc`` or ``asc`` after the sort criteria. For example, ``spec asc``.Possible values: ``id``, ``identifier``, ``title``, ``description``, ``spec``, ``spec.identifier``.
        sort: Optional[str] = None

    
    @dataclass
    class PackagesRequestBuilderGetRequestConfiguration(RequestConfiguration[PackagesRequestBuilderGetQueryParameters]):
        """
        Configuration for the request such as headers, query parameters, and middleware options.
        """
        warn("This class is deprecated. Please use the generic RequestConfiguration class generated by the generator.", DeprecationWarning)
    




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union

@dataclass
class ReviewSheetsPatchRequestBody(Parsable):
    # - ``true`` if you want to delete the review sheet.- ``false`` if you want to restore the review sheet.Note that if the review sheet has been deleted, it will not be published.
    deleted: Optional[bool] = None
    # The ID of the review sheet to update.
    id: Optional[str] = None
    # The new number of the review sheet. Note the following limitations:- The number should not contain these reserved characters: ``<``, ``>``, ``:``, ``````, ``/``, ````, ``|``, ``?``, ``*``, ``n``, ``r``, ``t``, ``0``, ``f``, ``'``.- You cannot assign the following reserved names to the number: CON, PRN, AUX, NUL, COM1, COM2, COM3, COM4, COM5, COM6, COM7, COM8, COM9, LPT1, LPT2, LPT3, LPT4, LPT5, LPT6, LPT7, LPT8, and LPT9.- You cannot put a period at the end of the number.- The number should not be space only.- The max length is 255.The API will format the number in the following ways before applying it to the review sheet:- Remove spaces at the end and beginning of the number.- Reduce multiple continuous spaces to a single space.Max length: 255
    number: Optional[str] = None
    # The new tags of the review sheet.- The max length is 100.- The tags should not be space only.- The max number of items is 50.The API will format the tags in the following ways before applying them to the review sheet:- Remove spaces at the end and beginning of the tags.- Reduce multiple continuous spaces to a single space.- The tags are case insensitive. Upper case letters will be transformed to lower case.Note that the tags that you specify overwrite existing tags.
    tags: Optional[list[str]] = None
    # The new title of the review sheet.- The title should not be space only.- The max length is 255.The API will format the title in the following ways before applying it to the review sheet:- Remove spaces at the end and beginning of the title.- Reduce multiple continuous spaces to a single space.Max length: 255
    title: Optional[str] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> ReviewSheetsPatchRequestBody:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: ReviewSheetsPatchRequestBody
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return ReviewSheetsPatchRequestBody()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        fields: dict[str, Callable[[Any], None]] = {
            "deleted": lambda n : setattr(self, 'deleted', n.get_bool_value()),
            "id": lambda n : setattr(self, 'id', n.get_str_value()),
            "number": lambda n : setattr(self, 'number', n.get_str_value()),
            "tags": lambda n : setattr(self, 'tags', n.get_collection_of_primitive_values(str)),
            "title": lambda n : setattr(self, 'title', n.get_str_value()),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_bool_value("deleted", self.deleted)
        writer.write_str_value("id", self.id)
        writer.write_str_value("number", self.number)
        writer.write_collection_of_primitive_values("tags", self.tags)
        writer.write_str_value("title", self.title)
    




from __future__ import annotations
import datetime
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union
from uuid import UUID

if TYPE_CHECKING:
    from .payments_get_response_additional_collaborators import PaymentsGetResponse_additionalCollaborators
    from .payments_get_response_association_type import PaymentsGetResponse_associationType
    from .payments_get_response_recipients import PaymentsGetResponse_recipients
    from .payments_get_response_status import PaymentsGetResponse_status

@dataclass
class PaymentsGetResponse(Parsable):
    # The additional collaborator company and contacts.
    additional_collaborators: Optional[list[PaymentsGetResponse_additionalCollaborators]] = None
    # The advance amount of the work completed in this period.
    advance_amount: Optional[float] = None
    # The advance percent of the work completed in this period.
    advance_percent: Optional[float] = None
    # Not relevant
    aggregate_by: Optional[str] = None
    # The amount of the work completed in this period.
    amount: Optional[float] = None
    # The date and time that the item was approved, in ISO 8601 format.
    approved_at: Optional[str] = None
    # Not relevant
    approved_budget_transfers: Optional[float] = None
    # Not relevant
    approved_budget_transfers_foreign_currency: Optional[float] = None
    # The amount of total approved change orders in this payment.
    approved_change_orders: Optional[float] = None
    # The amount of total approved change orders in this payment, in foreign currency.
    approved_change_orders_foreign_currency: Optional[float] = None
    # The BIM360/ACC ID of the architecture firm. The detailed company information can be retrieved from `GET projects/:project_id/companies <en/docs/bim360/v1/reference/http/projects-:project_id-companies-GET/>`_ endpoint by searching the `member_group_id` in the response.
    architect_company_id: Optional[str] = None
    # The unique ID of the architecture firm. The detailed company information can be retrieved by this uuid from `GET companies/:company_id <en/docs/bim360/v1/reference/http/companies-:company_id-GET/>` in the response
    architect_company_uid: Optional[str] = None
    # The BIM360/ACC ID of the primary contact at the architecture firm.
    architect_contact_id: Optional[str] = None
    # The ID of the contract or main contract that the payment is associated with.
    association_id: Optional[UUID] = None
    # The type of contract associated with this payment.Possible values: ``Contract``, ``MainContract``.
    association_type: Optional[PaymentsGetResponse_associationType] = None
    # Not relevant
    billing_period_id: Optional[UUID] = None
    # The ID of the budget payment associated with this cost payment. This field is valid only for cost payments.
    budget_payment_id: Optional[str] = None
    # Not relevant
    budget_transfer_ids: Optional[list[str]] = None
    # The date and time that the item was calculated, in ISO 8601 format.
    calculated_at: Optional[str] = None
    # The ID of the user who calculated the payment.
    calculated_by: Optional[str] = None
    # Not relevant
    can_release_retention: Optional[bool] = None
    # The claimed amount of total work completed in this period.
    claimed_amount: Optional[float] = None
    # The ID of a company managed by a BIM 360 Admin. Detailed company information can be retrieved by calling `GET projects/:project_id/companies <en/docs/bim360/v1/reference/http/projects-:project_id-companies-GET/>`_ and locating the `member_group_id` in the response.
    company_id: Optional[str] = None
    # The unique ID (UUID) of the company in this account. Detailed company information can be retrieved using this UUID by calling `GET companies/:company_id <en/docs/bim360/v1/reference/http/companies-:company_id-GET/>`_ in the response.
    company_uid: Optional[str] = None
    # The percentage of the gross retention amount of the payment for work completed. (released value is not removed)
    completed_work_gross_retention_percent: Optional[float] = None
    # Not relevant
    completed_work_retained: Optional[float] = None
    # Not relevant
    completed_work_retained_percent: Optional[float] = None
    # The amount of total work completed retained in this application.
    completed_work_retention: Optional[float] = None
    # The amount of total work completed retained (pre-release), in foreign currency.
    completed_work_retention_foreign_currency: Optional[float] = None
    # The ID of a contact managed by a BIM 360 Admin.
    contact_id: Optional[str] = None
    # The original contract amount of the payment.
    contract_amount: Optional[float] = None
    # The original contract amount of the payment, in foreign currency.
    contract_amount_foreign_currency: Optional[float] = None
    # The date and time that the item was created, in ISO 8601 format.
    created_at: Optional[datetime.datetime] = None
    # The user who created the payment. This is the ID of a user managed by BIM 360 Admin.
    creator_id: Optional[str] = None
    # Not relevant
    current_completed_work_retention_percent: Optional[float] = None
    # Not relevant
    current_materials_retention_percent: Optional[float] = None
    # A detailed description of this payment.
    description: Optional[str] = None
    # The due date of this payment.
    due_date: Optional[str] = None
    # The end date of this payment.
    end_date: Optional[str] = None
    # Exchange rate. Default value is ``1``. If multi-currency is not enabled, it will also be ``1``.
    exchange_rate: Optional[float] = None
    # The identifier assigned to an item in its original external ERP system. Use this ID to track and look up data within the integrated system. Note that this value comes from the item's ID in the external system.Max length: 255
    external_id: Optional[str] = None
    # A message generated by the external ERP system that explains the sync status of the integration. For example, common values include ``success`` or ``fail`` to indicate the result of the integration operation.Max length: 255
    external_message: Optional[str] = None
    # The name of the external ERP system integrated with Cost Management. Use this name to identify and search for data within the integrated system.Max length: 255
    external_system: Optional[str] = None
    # The date and time determine which distribution period this payment belongs to, after the status of ``status`` is set to ``approved`` or ``paid``.
    forecast_distribution_at: Optional[str] = None
    # Indicate if this payment has item rejected. This parameter is deprecated.
    has_item_rejected: Optional[bool] = None
    # Indicate if this payment has item reviewed. This parameter is deprecated.
    has_item_reviewed: Optional[bool] = None
    # The unique identifier of the payment.
    id: Optional[UUID] = None
    # The state of the item during the integration with the external ERP system (such as SignNow). An item can be a ``budget``, ``contract``, ``main contract``, ``main contract item``, ``cost item``, ``expense``, ``expense item``, ``change order``, or ``schedule of value``. For more details, see `Integrate with External System </en/docs/acc/v1/tutorials/cost/integrate-with-external-system/>`_ tutorial.Possible values:``locked``: the item is currently locked within the ERP system, preventing modifications until unlocked. To unlock and modify the item, use the relevant PATCH endpoint to set ``integrationState`` to ``null``. For example, for a budget, call `PATCH budgets <en/docs/bim360/v1/reference/http/cost-budgets-budgetId-PATCH/>`_. For a contract, call `PATCH contracts <en/docs/bim360/v1/reference/http/cost-contracts-contractId-PATCH/>`_. For more details, see the `Help documentation <https://help.autodesk.com/view/BUILD/ENU/?guid=Integrated_and_Locked>`_.``integrated``: the item has been successfully added to the ERP system.``failed``: the item encountered an error during the integration process and was not successfully added to the ERP system. For example, if a user tries to integrate ``contracts`` from an ERP system and the updates fail, the ``integrationState`` can be set to ``failed``. Retry the sync process or analyze the issue if it continues to fail.``null``: The item has not been integrated with the ERP system. This is default value.For more information regarding integrations within the Cost Management system, see `Integrations in Cost Management <https://help.autodesk.com/view/BUILD/ENU/?guid=Cost_Integrations>`_.
    integration_state: Optional[str] = None
    # The date and time that the item's integration status was last changed.
    integration_state_changed_at: Optional[str] = None
    # The user who last changed the integration status. This is the ID of a user managed by the BIM 360/ACC Admin.
    integration_state_changed_by: Optional[str] = None
    # Not relevant
    internal_id: Optional[str] = None
    # Not relevant
    internal_system: Optional[str] = None
    # The date and time when the item was last synchronized with the external ERP system. This value is updated by the external system and is in ISO 8601 format.
    last_sync_time: Optional[datetime.datetime] = None
    # The ID of the main contract associated with this payment.
    main_contract_id: Optional[UUID] = None
    # The amount of the materials billed in this period.
    materials_billed: Optional[float] = None
    # The percentage of the gross retention of the payment for material (released value is not removed).
    materials_gross_retention_percent: Optional[float] = None
    # The amount of the materials stored in this application.
    materials_on_store: Optional[float] = None
    # The amount of the materials stored retained in this application. This parameter is deprecated.
    materials_on_store_retention: Optional[float] = None
    # The amount of the materials (stored or billed) retained in this application.
    materials_retained: Optional[float] = None
    # Not relevant
    materials_retained_percent: Optional[float] = None
    # The amount of the materials (stored or billed) retained in this application.
    materials_retention: Optional[float] = None
    # The amount of the materials (stored or billed) retained in this application, in foreign currency.
    materials_retention_foreign_currency: Optional[float] = None
    # The name of this payment.Max length: 1024
    name: Optional[str] = None
    # The net amount that should be paid in this period, including work completed, advance amount, materials stored or billed, retention and release.
    net_amount: Optional[float] = None
    # The net amount that should be paid in this period, including work completed, advance amount, materials stored or billed, retention and release, in foreign currency.
    net_amount_foreign_currency: Optional[float] = None
    # The net materials stored in this period, which equals to (`materialsOnStore` - `previousMaterialsOnStore`).
    net_materials_on_store: Optional[float] = None
    # The net retention in this period, which equals to (`completedWorkRetention` + `materialsOnStoreRetention` - `previousRetention`).
    net_retention: Optional[float] = None
    # The note attached to the payment, if returned. The note consists of Tiptap formatted rich text. For more information, see `https://tiptap.dev <https://tiptap.dev/introduction/>`_.
    note: Optional[str] = None
    # The number of this payment.Max length: 255
    number: Optional[str] = None
    # The original payment amount, including the main contract amount, change orders, and internal budget transfers that affect the schedule of values defined in project settings.
    original_amount: Optional[float] = None
    # The original payment amount in foreign currency, including the main contract amount, change orders, and internal budget transfers that affect the schedule of values defined in project settings.
    original_amount_foreign_currency: Optional[float] = None
    # The date and time when the expense is paid.
    paid_at: Optional[str] = None
    # Not relevant
    payment_stage: Optional[str] = None
    # The advance amount of total work completed prior to this period.
    previous_advance_amount: Optional[float] = None
    # The advance amount of total work completed prior to this period, in foreign currency.
    previous_advance_amount_foreign_currency: Optional[float] = None
    # The amount of total work completed prior to this period.
    previous_amount: Optional[float] = None
    # The amount of total work completed prior to this period, in foreign currency.
    previous_amount_foreign_currency: Optional[float] = None
    # Not relevant
    previous_approved_budget_transfers: Optional[float] = None
    # Not relevant
    previous_approved_budget_transfers_foreign_currency: Optional[float] = None
    # The amount of total approved change orders prior to this period.
    previous_approved_change_orders: Optional[float] = None
    # The amount of total approved change orders prior to this period, in foreign currency.
    previous_approved_change_orders_foreign_currency: Optional[float] = None
    # The claimed amount of total work completed prior to this period.
    previous_claimed_amount: Optional[float] = None
    # The claimed amount of total work completed prior to this period, in foreign currency.
    previous_claimed_amount_foreign_currency: Optional[float] = None
    # Exchange rate from previous payment application. Default value is ``1``, if multi-currency is not enabled, it will also be ``1``.
    previous_exchange_rate: Optional[float] = None
    # The amount of total materials billed prior to this period.
    previous_materials_billed: Optional[float] = None
    # The amount of total materials billed prior to this period, in foreign currency.
    previous_materials_billed_foreign_currency: Optional[float] = None
    # The amount of total materials stored prior to this period.
    previous_materials_on_store: Optional[float] = None
    # The unique identifier of the previous payment for this payment.
    previous_payment_id: Optional[UUID] = None
    # The recoupment amount of total work completed prior to this period.
    previous_recoupment_amount: Optional[float] = None
    # The recoupment amount of total work completed prior to this period, in foreign currency.
    previous_recoupment_amount_foreign_currency: Optional[float] = None
    # The retained amount of total work completed and materials (stored or billed) prior to this period.
    previous_retention: Optional[float] = None
    # The retained amount of total work completed and materials (stored or billed) prior to this period, in foreign currency.
    previous_retention_foreign_currency: Optional[float] = None
    # Persons that the generated documents will be sent to.
    recipients: Optional[list[PaymentsGetResponse_recipients]] = None
    # The recoupment amount of the work completed in this period.
    recoupment_amount: Optional[float] = None
    # The recoupment percentage of the work completed in this period.
    recoupment_percent_of_completed_work: Optional[float] = None
    # The start date of this payment.
    start_date: Optional[str] = None
    # The status of this payment. For details about the possible values of this field, see `Payment Application Statuses <https://help.autodesk.com/view/BUILD/ENU/?guid=Cost_Payment_Application_Statuses>`_.Possible values: ``draft``, ``pendingInput``, ``submitted``, ``revise``, ``accepted``, ``approved``, ``paid``.
    status: Optional[PaymentsGetResponse_status] = None
    # The date and time when the payment is submitted.
    submitted_at: Optional[str] = None
    # The percentage of the gross retention of the payment for work completed and material. (released value is not removed)
    total_gross_retention_percent: Optional[float] = None
    # Not relevant
    total_retained: Optional[float] = None
    # Not relevant
    total_retained_percent: Optional[float] = None
    # The date and time that the item was last updated, in ISO 8601 format.
    updated_at: Optional[datetime.datetime] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> PaymentsGetResponse:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: PaymentsGetResponse
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return PaymentsGetResponse()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        from .payments_get_response_additional_collaborators import PaymentsGetResponse_additionalCollaborators
        from .payments_get_response_association_type import PaymentsGetResponse_associationType
        from .payments_get_response_recipients import PaymentsGetResponse_recipients
        from .payments_get_response_status import PaymentsGetResponse_status

        from .payments_get_response_additional_collaborators import PaymentsGetResponse_additionalCollaborators
        from .payments_get_response_association_type import PaymentsGetResponse_associationType
        from .payments_get_response_recipients import PaymentsGetResponse_recipients
        from .payments_get_response_status import PaymentsGetResponse_status

        fields: dict[str, Callable[[Any], None]] = {
            "additionalCollaborators": lambda n : setattr(self, 'additional_collaborators', n.get_collection_of_object_values(PaymentsGetResponse_additionalCollaborators)),
            "advanceAmount": lambda n : setattr(self, 'advance_amount', n.get_float_value()),
            "advancePercent": lambda n : setattr(self, 'advance_percent', n.get_float_value()),
            "aggregateBy": lambda n : setattr(self, 'aggregate_by', n.get_str_value()),
            "amount": lambda n : setattr(self, 'amount', n.get_float_value()),
            "approvedAt": lambda n : setattr(self, 'approved_at', n.get_str_value()),
            "approvedBudgetTransfers": lambda n : setattr(self, 'approved_budget_transfers', n.get_float_value()),
            "approvedBudgetTransfersForeignCurrency": lambda n : setattr(self, 'approved_budget_transfers_foreign_currency', n.get_float_value()),
            "approvedChangeOrders": lambda n : setattr(self, 'approved_change_orders', n.get_float_value()),
            "approvedChangeOrdersForeignCurrency": lambda n : setattr(self, 'approved_change_orders_foreign_currency', n.get_float_value()),
            "architectCompanyId": lambda n : setattr(self, 'architect_company_id', n.get_str_value()),
            "architectCompanyUid": lambda n : setattr(self, 'architect_company_uid', n.get_str_value()),
            "architectContactId": lambda n : setattr(self, 'architect_contact_id', n.get_str_value()),
            "associationId": lambda n : setattr(self, 'association_id', n.get_uuid_value()),
            "associationType": lambda n : setattr(self, 'association_type', n.get_enum_value(PaymentsGetResponse_associationType)),
            "billingPeriodId": lambda n : setattr(self, 'billing_period_id', n.get_uuid_value()),
            "budgetPaymentId": lambda n : setattr(self, 'budget_payment_id', n.get_str_value()),
            "budgetTransferIds": lambda n : setattr(self, 'budget_transfer_ids', n.get_collection_of_primitive_values(str)),
            "calculatedAt": lambda n : setattr(self, 'calculated_at', n.get_str_value()),
            "calculatedBy": lambda n : setattr(self, 'calculated_by', n.get_str_value()),
            "canReleaseRetention": lambda n : setattr(self, 'can_release_retention', n.get_bool_value()),
            "claimedAmount": lambda n : setattr(self, 'claimed_amount', n.get_float_value()),
            "companyId": lambda n : setattr(self, 'company_id', n.get_str_value()),
            "companyUid": lambda n : setattr(self, 'company_uid', n.get_str_value()),
            "completedWorkGrossRetentionPercent": lambda n : setattr(self, 'completed_work_gross_retention_percent', n.get_float_value()),
            "completedWorkRetained": lambda n : setattr(self, 'completed_work_retained', n.get_float_value()),
            "completedWorkRetainedPercent": lambda n : setattr(self, 'completed_work_retained_percent', n.get_float_value()),
            "completedWorkRetention": lambda n : setattr(self, 'completed_work_retention', n.get_float_value()),
            "completedWorkRetentionForeignCurrency": lambda n : setattr(self, 'completed_work_retention_foreign_currency', n.get_float_value()),
            "contactId": lambda n : setattr(self, 'contact_id', n.get_str_value()),
            "contractAmount": lambda n : setattr(self, 'contract_amount', n.get_float_value()),
            "contractAmountForeignCurrency": lambda n : setattr(self, 'contract_amount_foreign_currency', n.get_float_value()),
            "createdAt": lambda n : setattr(self, 'created_at', n.get_datetime_value()),
            "creatorId": lambda n : setattr(self, 'creator_id', n.get_str_value()),
            "currentCompletedWorkRetentionPercent": lambda n : setattr(self, 'current_completed_work_retention_percent', n.get_float_value()),
            "currentMaterialsRetentionPercent": lambda n : setattr(self, 'current_materials_retention_percent', n.get_float_value()),
            "description": lambda n : setattr(self, 'description', n.get_str_value()),
            "dueDate": lambda n : setattr(self, 'due_date', n.get_str_value()),
            "endDate": lambda n : setattr(self, 'end_date', n.get_str_value()),
            "exchangeRate": lambda n : setattr(self, 'exchange_rate', n.get_float_value()),
            "externalId": lambda n : setattr(self, 'external_id', n.get_str_value()),
            "externalMessage": lambda n : setattr(self, 'external_message', n.get_str_value()),
            "externalSystem": lambda n : setattr(self, 'external_system', n.get_str_value()),
            "forecastDistributionAt": lambda n : setattr(self, 'forecast_distribution_at', n.get_str_value()),
            "hasItemRejected": lambda n : setattr(self, 'has_item_rejected', n.get_bool_value()),
            "hasItemReviewed": lambda n : setattr(self, 'has_item_reviewed', n.get_bool_value()),
            "id": lambda n : setattr(self, 'id', n.get_uuid_value()),
            "integrationState": lambda n : setattr(self, 'integration_state', n.get_str_value()),
            "integrationStateChangedAt": lambda n : setattr(self, 'integration_state_changed_at', n.get_str_value()),
            "integrationStateChangedBy": lambda n : setattr(self, 'integration_state_changed_by', n.get_str_value()),
            "internalId": lambda n : setattr(self, 'internal_id', n.get_str_value()),
            "internalSystem": lambda n : setattr(self, 'internal_system', n.get_str_value()),
            "lastSyncTime": lambda n : setattr(self, 'last_sync_time', n.get_datetime_value()),
            "mainContractId": lambda n : setattr(self, 'main_contract_id', n.get_uuid_value()),
            "materialsBilled": lambda n : setattr(self, 'materials_billed', n.get_float_value()),
            "materialsGrossRetentionPercent": lambda n : setattr(self, 'materials_gross_retention_percent', n.get_float_value()),
            "materialsOnStore": lambda n : setattr(self, 'materials_on_store', n.get_float_value()),
            "materialsOnStoreRetention": lambda n : setattr(self, 'materials_on_store_retention', n.get_float_value()),
            "materialsRetained": lambda n : setattr(self, 'materials_retained', n.get_float_value()),
            "materialsRetainedPercent": lambda n : setattr(self, 'materials_retained_percent', n.get_float_value()),
            "materialsRetention": lambda n : setattr(self, 'materials_retention', n.get_float_value()),
            "materialsRetentionForeignCurrency": lambda n : setattr(self, 'materials_retention_foreign_currency', n.get_float_value()),
            "name": lambda n : setattr(self, 'name', n.get_str_value()),
            "netAmount": lambda n : setattr(self, 'net_amount', n.get_float_value()),
            "netAmountForeignCurrency": lambda n : setattr(self, 'net_amount_foreign_currency', n.get_float_value()),
            "netMaterialsOnStore": lambda n : setattr(self, 'net_materials_on_store', n.get_float_value()),
            "netRetention": lambda n : setattr(self, 'net_retention', n.get_float_value()),
            "note": lambda n : setattr(self, 'note', n.get_str_value()),
            "number": lambda n : setattr(self, 'number', n.get_str_value()),
            "originalAmount": lambda n : setattr(self, 'original_amount', n.get_float_value()),
            "originalAmountForeignCurrency": lambda n : setattr(self, 'original_amount_foreign_currency', n.get_float_value()),
            "paidAt": lambda n : setattr(self, 'paid_at', n.get_str_value()),
            "paymentStage": lambda n : setattr(self, 'payment_stage', n.get_str_value()),
            "previousAdvanceAmount": lambda n : setattr(self, 'previous_advance_amount', n.get_float_value()),
            "previousAdvanceAmountForeignCurrency": lambda n : setattr(self, 'previous_advance_amount_foreign_currency', n.get_float_value()),
            "previousAmount": lambda n : setattr(self, 'previous_amount', n.get_float_value()),
            "previousAmountForeignCurrency": lambda n : setattr(self, 'previous_amount_foreign_currency', n.get_float_value()),
            "previousApprovedBudgetTransfers": lambda n : setattr(self, 'previous_approved_budget_transfers', n.get_float_value()),
            "previousApprovedBudgetTransfersForeignCurrency": lambda n : setattr(self, 'previous_approved_budget_transfers_foreign_currency', n.get_float_value()),
            "previousApprovedChangeOrders": lambda n : setattr(self, 'previous_approved_change_orders', n.get_float_value()),
            "previousApprovedChangeOrdersForeignCurrency": lambda n : setattr(self, 'previous_approved_change_orders_foreign_currency', n.get_float_value()),
            "previousClaimedAmount": lambda n : setattr(self, 'previous_claimed_amount', n.get_float_value()),
            "previousClaimedAmountForeignCurrency": lambda n : setattr(self, 'previous_claimed_amount_foreign_currency', n.get_float_value()),
            "previousExchangeRate": lambda n : setattr(self, 'previous_exchange_rate', n.get_float_value()),
            "previousMaterialsBilled": lambda n : setattr(self, 'previous_materials_billed', n.get_float_value()),
            "previousMaterialsBilledForeignCurrency": lambda n : setattr(self, 'previous_materials_billed_foreign_currency', n.get_float_value()),
            "previousMaterialsOnStore": lambda n : setattr(self, 'previous_materials_on_store', n.get_float_value()),
            "previousPaymentId": lambda n : setattr(self, 'previous_payment_id', n.get_uuid_value()),
            "previousRecoupmentAmount": lambda n : setattr(self, 'previous_recoupment_amount', n.get_float_value()),
            "previousRecoupmentAmountForeignCurrency": lambda n : setattr(self, 'previous_recoupment_amount_foreign_currency', n.get_float_value()),
            "previousRetention": lambda n : setattr(self, 'previous_retention', n.get_float_value()),
            "previousRetentionForeignCurrency": lambda n : setattr(self, 'previous_retention_foreign_currency', n.get_float_value()),
            "recipients": lambda n : setattr(self, 'recipients', n.get_collection_of_object_values(PaymentsGetResponse_recipients)),
            "recoupmentAmount": lambda n : setattr(self, 'recoupment_amount', n.get_float_value()),
            "recoupmentPercentOfCompletedWork": lambda n : setattr(self, 'recoupment_percent_of_completed_work', n.get_float_value()),
            "startDate": lambda n : setattr(self, 'start_date', n.get_str_value()),
            "status": lambda n : setattr(self, 'status', n.get_enum_value(PaymentsGetResponse_status)),
            "submittedAt": lambda n : setattr(self, 'submitted_at', n.get_str_value()),
            "totalGrossRetentionPercent": lambda n : setattr(self, 'total_gross_retention_percent', n.get_float_value()),
            "totalRetained": lambda n : setattr(self, 'total_retained', n.get_float_value()),
            "totalRetainedPercent": lambda n : setattr(self, 'total_retained_percent', n.get_float_value()),
            "updatedAt": lambda n : setattr(self, 'updated_at', n.get_datetime_value()),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_collection_of_object_values("additionalCollaborators", self.additional_collaborators)
        writer.write_float_value("advanceAmount", self.advance_amount)
        writer.write_float_value("advancePercent", self.advance_percent)
        writer.write_str_value("aggregateBy", self.aggregate_by)
        writer.write_float_value("amount", self.amount)
        writer.write_str_value("approvedAt", self.approved_at)
        writer.write_float_value("approvedBudgetTransfers", self.approved_budget_transfers)
        writer.write_float_value("approvedBudgetTransfersForeignCurrency", self.approved_budget_transfers_foreign_currency)
        writer.write_float_value("approvedChangeOrders", self.approved_change_orders)
        writer.write_float_value("approvedChangeOrdersForeignCurrency", self.approved_change_orders_foreign_currency)
        writer.write_str_value("architectCompanyId", self.architect_company_id)
        writer.write_str_value("architectCompanyUid", self.architect_company_uid)
        writer.write_str_value("architectContactId", self.architect_contact_id)
        writer.write_uuid_value("associationId", self.association_id)
        writer.write_enum_value("associationType", self.association_type)
        writer.write_uuid_value("billingPeriodId", self.billing_period_id)
        writer.write_str_value("budgetPaymentId", self.budget_payment_id)
        writer.write_collection_of_primitive_values("budgetTransferIds", self.budget_transfer_ids)
        writer.write_str_value("calculatedAt", self.calculated_at)
        writer.write_str_value("calculatedBy", self.calculated_by)
        writer.write_bool_value("canReleaseRetention", self.can_release_retention)
        writer.write_float_value("claimedAmount", self.claimed_amount)
        writer.write_str_value("companyId", self.company_id)
        writer.write_str_value("companyUid", self.company_uid)
        writer.write_float_value("completedWorkGrossRetentionPercent", self.completed_work_gross_retention_percent)
        writer.write_float_value("completedWorkRetained", self.completed_work_retained)
        writer.write_float_value("completedWorkRetainedPercent", self.completed_work_retained_percent)
        writer.write_float_value("completedWorkRetention", self.completed_work_retention)
        writer.write_float_value("completedWorkRetentionForeignCurrency", self.completed_work_retention_foreign_currency)
        writer.write_str_value("contactId", self.contact_id)
        writer.write_float_value("contractAmount", self.contract_amount)
        writer.write_float_value("contractAmountForeignCurrency", self.contract_amount_foreign_currency)
        writer.write_datetime_value("createdAt", self.created_at)
        writer.write_str_value("creatorId", self.creator_id)
        writer.write_float_value("currentCompletedWorkRetentionPercent", self.current_completed_work_retention_percent)
        writer.write_float_value("currentMaterialsRetentionPercent", self.current_materials_retention_percent)
        writer.write_str_value("description", self.description)
        writer.write_str_value("dueDate", self.due_date)
        writer.write_str_value("endDate", self.end_date)
        writer.write_float_value("exchangeRate", self.exchange_rate)
        writer.write_str_value("externalId", self.external_id)
        writer.write_str_value("externalMessage", self.external_message)
        writer.write_str_value("externalSystem", self.external_system)
        writer.write_str_value("forecastDistributionAt", self.forecast_distribution_at)
        writer.write_bool_value("hasItemRejected", self.has_item_rejected)
        writer.write_bool_value("hasItemReviewed", self.has_item_reviewed)
        writer.write_uuid_value("id", self.id)
        writer.write_str_value("integrationState", self.integration_state)
        writer.write_str_value("integrationStateChangedAt", self.integration_state_changed_at)
        writer.write_str_value("integrationStateChangedBy", self.integration_state_changed_by)
        writer.write_str_value("internalId", self.internal_id)
        writer.write_str_value("internalSystem", self.internal_system)
        writer.write_datetime_value("lastSyncTime", self.last_sync_time)
        writer.write_uuid_value("mainContractId", self.main_contract_id)
        writer.write_float_value("materialsBilled", self.materials_billed)
        writer.write_float_value("materialsGrossRetentionPercent", self.materials_gross_retention_percent)
        writer.write_float_value("materialsOnStore", self.materials_on_store)
        writer.write_float_value("materialsOnStoreRetention", self.materials_on_store_retention)
        writer.write_float_value("materialsRetained", self.materials_retained)
        writer.write_float_value("materialsRetainedPercent", self.materials_retained_percent)
        writer.write_float_value("materialsRetention", self.materials_retention)
        writer.write_float_value("materialsRetentionForeignCurrency", self.materials_retention_foreign_currency)
        writer.write_str_value("name", self.name)
        writer.write_float_value("netAmount", self.net_amount)
        writer.write_float_value("netAmountForeignCurrency", self.net_amount_foreign_currency)
        writer.write_float_value("netMaterialsOnStore", self.net_materials_on_store)
        writer.write_float_value("netRetention", self.net_retention)
        writer.write_str_value("note", self.note)
        writer.write_str_value("number", self.number)
        writer.write_float_value("originalAmount", self.original_amount)
        writer.write_float_value("originalAmountForeignCurrency", self.original_amount_foreign_currency)
        writer.write_str_value("paidAt", self.paid_at)
        writer.write_str_value("paymentStage", self.payment_stage)
        writer.write_float_value("previousAdvanceAmount", self.previous_advance_amount)
        writer.write_float_value("previousAdvanceAmountForeignCurrency", self.previous_advance_amount_foreign_currency)
        writer.write_float_value("previousAmount", self.previous_amount)
        writer.write_float_value("previousAmountForeignCurrency", self.previous_amount_foreign_currency)
        writer.write_float_value("previousApprovedBudgetTransfers", self.previous_approved_budget_transfers)
        writer.write_float_value("previousApprovedBudgetTransfersForeignCurrency", self.previous_approved_budget_transfers_foreign_currency)
        writer.write_float_value("previousApprovedChangeOrders", self.previous_approved_change_orders)
        writer.write_float_value("previousApprovedChangeOrdersForeignCurrency", self.previous_approved_change_orders_foreign_currency)
        writer.write_float_value("previousClaimedAmount", self.previous_claimed_amount)
        writer.write_float_value("previousClaimedAmountForeignCurrency", self.previous_claimed_amount_foreign_currency)
        writer.write_float_value("previousExchangeRate", self.previous_exchange_rate)
        writer.write_float_value("previousMaterialsBilled", self.previous_materials_billed)
        writer.write_float_value("previousMaterialsBilledForeignCurrency", self.previous_materials_billed_foreign_currency)
        writer.write_float_value("previousMaterialsOnStore", self.previous_materials_on_store)
        writer.write_uuid_value("previousPaymentId", self.previous_payment_id)
        writer.write_float_value("previousRecoupmentAmount", self.previous_recoupment_amount)
        writer.write_float_value("previousRecoupmentAmountForeignCurrency", self.previous_recoupment_amount_foreign_currency)
        writer.write_float_value("previousRetention", self.previous_retention)
        writer.write_float_value("previousRetentionForeignCurrency", self.previous_retention_foreign_currency)
        writer.write_collection_of_object_values("recipients", self.recipients)
        writer.write_float_value("recoupmentAmount", self.recoupment_amount)
        writer.write_float_value("recoupmentPercentOfCompletedWork", self.recoupment_percent_of_completed_work)
        writer.write_str_value("startDate", self.start_date)
        writer.write_enum_value("status", self.status)
        writer.write_str_value("submittedAt", self.submitted_at)
        writer.write_float_value("totalGrossRetentionPercent", self.total_gross_retention_percent)
        writer.write_float_value("totalRetained", self.total_retained)
        writer.write_float_value("totalRetainedPercent", self.total_retained_percent)
        writer.write_datetime_value("updatedAt", self.updated_at)
    




from __future__ import annotations
from collections.abc import Callable
from kiota_abstractions.base_request_builder import BaseRequestBuilder
from kiota_abstractions.get_path_parameters import get_path_parameters
from kiota_abstractions.request_adapter import RequestAdapter
from typing import Any, Optional, TYPE_CHECKING, Union

if TYPE_CHECKING:
    from .item.with_tree_item_request_builder import WithTreeItemRequestBuilder

class TreesRequestBuilder(BaseRequestBuilder):
    """
    Builds and executes requests for operations under /construction/locations/v2/projects/{projectId}/trees
    """
    def __init__(self,request_adapter: RequestAdapter, path_parameters: Union[str, dict[str, Any]]) -> None:
        """
        Instantiates a new TreesRequestBuilder and sets the default values.
        param path_parameters: The raw url or the url-template parameters for the request.
        param request_adapter: The request adapter to use to execute the requests.
        Returns: None
        """
        super().__init__(request_adapter, "{+baseurl}/construction/locations/v2/projects/{projectId}/trees", path_parameters)
    
    def by_tree_id(self,tree_id: str) -> WithTreeItemRequestBuilder:
        """
        Gets an item from the Autodesk.ACC.construction.locations.v2.projects.item.trees.item collection
        param tree_id: Must be ``default``. Currently a project can contain only the default tree.
        Returns: WithTreeItemRequestBuilder
        """
        if tree_id is None:
            raise TypeError("tree_id cannot be null.")
        from .item.with_tree_item_request_builder import WithTreeItemRequestBuilder

        url_tpl_params = get_path_parameters(self.path_parameters)
        url_tpl_params["treeId"] = tree_id
        return WithTreeItemRequestBuilder(self.request_adapter, url_tpl_params)
    




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union

if TYPE_CHECKING:
    from .with_response_patch_request_body_attachments import WithResponsePatchRequestBody_attachments
    from .with_response_patch_request_body_on_behalf import WithResponsePatchRequestBody_onBehalf
    from .with_response_patch_request_body_status import WithResponsePatchRequestBody_status

@dataclass
class WithResponsePatchRequestBody(Parsable):
    # A list of files to attach to the RFI response.
    attachments: Optional[list[WithResponsePatchRequestBody_attachments]] = None
    # The Autodesk ID of the reviewer for whom the response is being submitted.To find user details, call `GET users </en/docs/acc/v1/reference/http/admin-projectsprojectId-users-GET/>`_.'
    on_behalf: Optional[WithResponsePatchRequestBody_onBehalf] = None
    # The status of the response.To determine which status values the user is allowed to submit, call `GET users/me </en/docs/acc/v1/reference/http/rfis-users-me-GET/>`_.Possible values: ``answered``, ``rejected``.
    status: Optional[WithResponsePatchRequestBody_status] = None
    # The body text of the RFI response. This field supports plain text only.
    text: Optional[str] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> WithResponsePatchRequestBody:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: WithResponsePatchRequestBody
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return WithResponsePatchRequestBody()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        from .with_response_patch_request_body_attachments import WithResponsePatchRequestBody_attachments
        from .with_response_patch_request_body_on_behalf import WithResponsePatchRequestBody_onBehalf
        from .with_response_patch_request_body_status import WithResponsePatchRequestBody_status

        from .with_response_patch_request_body_attachments import WithResponsePatchRequestBody_attachments
        from .with_response_patch_request_body_on_behalf import WithResponsePatchRequestBody_onBehalf
        from .with_response_patch_request_body_status import WithResponsePatchRequestBody_status

        fields: dict[str, Callable[[Any], None]] = {
            "attachments": lambda n : setattr(self, 'attachments', n.get_collection_of_object_values(WithResponsePatchRequestBody_attachments)),
            "onBehalf": lambda n : setattr(self, 'on_behalf', n.get_object_value(WithResponsePatchRequestBody_onBehalf)),
            "status": lambda n : setattr(self, 'status', n.get_enum_value(WithResponsePatchRequestBody_status)),
            "text": lambda n : setattr(self, 'text', n.get_str_value()),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_collection_of_object_values("attachments", self.attachments)
        writer.write_object_value("onBehalf", self.on_behalf)
        writer.write_enum_value("status", self.status)
        writer.write_str_value("text", self.text)
    




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union

if TYPE_CHECKING:
    from .indexes_batch_status_post_request_body_versions_columns import IndexesBatchStatusPostRequestBody_versions_columns
    from .indexes_batch_status_post_request_body_versions_query import IndexesBatchStatusPostRequestBody_versions_query

@dataclass
class IndexesBatchStatusPostRequestBody_versions(Parsable):
    # `SQL AST for describing columns/projections </en/docs/acc/v1/tutorials/model-properties/query-ref/>`_
    columns: Optional[IndexesBatchStatusPostRequestBody_versions_columns] = None
    # `SQL AST for binary expression/filter </en/docs/acc/v1/tutorials/model-properties/query-ref/>`_
    query: Optional[IndexesBatchStatusPostRequestBody_versions_query] = None
    # file version urn.
    version_urn: Optional[str] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> IndexesBatchStatusPostRequestBody_versions:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: IndexesBatchStatusPostRequestBody_versions
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return IndexesBatchStatusPostRequestBody_versions()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        from .indexes_batch_status_post_request_body_versions_columns import IndexesBatchStatusPostRequestBody_versions_columns
        from .indexes_batch_status_post_request_body_versions_query import IndexesBatchStatusPostRequestBody_versions_query

        from .indexes_batch_status_post_request_body_versions_columns import IndexesBatchStatusPostRequestBody_versions_columns
        from .indexes_batch_status_post_request_body_versions_query import IndexesBatchStatusPostRequestBody_versions_query

        fields: dict[str, Callable[[Any], None]] = {
            "columns": lambda n : setattr(self, 'columns', n.get_object_value(IndexesBatchStatusPostRequestBody_versions_columns)),
            "query": lambda n : setattr(self, 'query', n.get_object_value(IndexesBatchStatusPostRequestBody_versions_query)),
            "versionUrn": lambda n : setattr(self, 'version_urn', n.get_str_value()),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_object_value("columns", self.columns)
        writer.write_object_value("query", self.query)
        writer.write_str_value("versionUrn", self.version_urn)
    




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union
from uuid import UUID

if TYPE_CHECKING:
    from .packages_get_response_results_permitted_actions import PackagesGetResponse_results_permittedActions

@dataclass
class PackagesGetResponse_results(Parsable):
    # The description of the package.
    description: Optional[str] = None
    # The internal, globally unique identifier (UUID) for the package.
    id: Optional[UUID] = None
    # The unique ID assigned to the package within the UI.
    identifier: Optional[int] = None
    # The list of actions the user is allowed to perform on the submittal item.
    permitted_actions: Optional[list[PackagesGetResponse_results_permittedActions]] = None
    # The internal, globally unique identifier (UUID) of the spec associated with the package.
    spec_id: Optional[UUID] = None
    # The unique ID of the spec assigned to the package in the UI, specific to each project.
    spec_identifier: Optional[str] = None
    # The title of the package.
    title: Optional[str] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> PackagesGetResponse_results:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: PackagesGetResponse_results
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return PackagesGetResponse_results()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        from .packages_get_response_results_permitted_actions import PackagesGetResponse_results_permittedActions

        from .packages_get_response_results_permitted_actions import PackagesGetResponse_results_permittedActions

        fields: dict[str, Callable[[Any], None]] = {
            "description": lambda n : setattr(self, 'description', n.get_str_value()),
            "id": lambda n : setattr(self, 'id', n.get_uuid_value()),
            "identifier": lambda n : setattr(self, 'identifier', n.get_int_value()),
            "permittedActions": lambda n : setattr(self, 'permitted_actions', n.get_collection_of_object_values(PackagesGetResponse_results_permittedActions)),
            "specId": lambda n : setattr(self, 'spec_id', n.get_uuid_value()),
            "specIdentifier": lambda n : setattr(self, 'spec_identifier', n.get_str_value()),
            "title": lambda n : setattr(self, 'title', n.get_str_value()),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_str_value("description", self.description)
        writer.write_uuid_value("id", self.id)
        writer.write_int_value("identifier", self.identifier)
        writer.write_collection_of_object_values("permittedActions", self.permitted_actions)
        writer.write_uuid_value("specId", self.spec_id)
        writer.write_str_value("specIdentifier", self.spec_identifier)
        writer.write_str_value("title", self.title)
    




from __future__ import annotations
from collections.abc import Callable
from kiota_abstractions.base_request_builder import BaseRequestBuilder
from kiota_abstractions.get_path_parameters import get_path_parameters
from kiota_abstractions.request_adapter import RequestAdapter
from typing import Any, Optional, TYPE_CHECKING, Union
from uuid import UUID

if TYPE_CHECKING:
    from .item.project_item_request_builder import ProjectItemRequestBuilder

class ProjectsRequestBuilder(BaseRequestBuilder):
    """
    Builds and executes requests for operations under /bim360/docs/v1/projects
    """
    def __init__(self,request_adapter: RequestAdapter, path_parameters: Union[str, dict[str, Any]]) -> None:
        """
        Instantiates a new ProjectsRequestBuilder and sets the default values.
        param path_parameters: The raw url or the url-template parameters for the request.
        param request_adapter: The request adapter to use to execute the requests.
        Returns: None
        """
        super().__init__(request_adapter, "{+baseurl}/bim360/docs/v1/projects", path_parameters)
    
    def by_project_id(self,project_id: UUID) -> ProjectItemRequestBuilder:
        """
        Gets an item from the Autodesk.ACC.bim360.docs.v1.projects.item collection
        param project_id: The ID of the project.This corresponds to project ID in the `Data Management API </en/docs/data/v2/>`_. To convert a project ID in the Data Management API into a project ID in the BIM 360 API you need to remove the ``**b.**`` prefix. For example, a project ID of **b.**a4be0c34a-4ab7 translates to a project ID of a4be0c34a-4ab7.To learn how to find the project ID, see the `Retrieve BIM 360 Account and Project ID </en/docs/bim360/v1/tutorials/getting-started/retrieve-account-and-project-id/>`_ tutorial.
        Returns: ProjectItemRequestBuilder
        """
        if project_id is None:
            raise TypeError("project_id cannot be null.")
        from .item.project_item_request_builder import ProjectItemRequestBuilder

        url_tpl_params = get_path_parameters(self.path_parameters)
        url_tpl_params["project%2Did"] = project_id
        return ProjectItemRequestBuilder(self.request_adapter, url_tpl_params)
    




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union

if TYPE_CHECKING:
    from .recipients_get_response_external_members import RecipientsGetResponse_externalMembers
    from .recipients_get_response_recipients import RecipientsGetResponse_recipients

@dataclass
class RecipientsGetResponse(Parsable):
    # A list of external recipients included in the transmittal.
    external_members: Optional[list[RecipientsGetResponse_externalMembers]] = None
    # The list of project-member recipients included in the transmittal.
    recipients: Optional[list[RecipientsGetResponse_recipients]] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> RecipientsGetResponse:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: RecipientsGetResponse
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return RecipientsGetResponse()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        from .recipients_get_response_external_members import RecipientsGetResponse_externalMembers
        from .recipients_get_response_recipients import RecipientsGetResponse_recipients

        from .recipients_get_response_external_members import RecipientsGetResponse_externalMembers
        from .recipients_get_response_recipients import RecipientsGetResponse_recipients

        fields: dict[str, Callable[[Any], None]] = {
            "externalMembers": lambda n : setattr(self, 'external_members', n.get_collection_of_object_values(RecipientsGetResponse_externalMembers)),
            "recipients": lambda n : setattr(self, 'recipients', n.get_collection_of_object_values(RecipientsGetResponse_recipients)),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_collection_of_object_values("externalMembers", self.external_members)
        writer.write_collection_of_object_values("recipients", self.recipients)
    




from __future__ import annotations
from collections.abc import Callable
from kiota_abstractions.base_request_builder import BaseRequestBuilder
from kiota_abstractions.get_path_parameters import get_path_parameters
from kiota_abstractions.request_adapter import RequestAdapter
from typing import Any, Optional, TYPE_CHECKING, Union
from uuid import UUID

if TYPE_CHECKING:
    from .item.with_project_item_request_builder import WithProjectItemRequestBuilder

class ProjectsRequestBuilder(BaseRequestBuilder):
    """
    Builds and executes requests for operations under /construction/transmittals/v1/projects
    """
    def __init__(self,request_adapter: RequestAdapter, path_parameters: Union[str, dict[str, Any]]) -> None:
        """
        Instantiates a new ProjectsRequestBuilder and sets the default values.
        param path_parameters: The raw url or the url-template parameters for the request.
        param request_adapter: The request adapter to use to execute the requests.
        Returns: None
        """
        super().__init__(request_adapter, "{+baseurl}/construction/transmittals/v1/projects", path_parameters)
    
    def by_project_id(self,project_id: UUID) -> WithProjectItemRequestBuilder:
        """
        Gets an item from the Autodesk.ACC.construction.transmittals.v1.projects.item collection
        param project_id: The ID of the project.You can retrieve the project ID using the `Data Management API </en/docs/data/v2/>`_. For more details, see the `Retrieve a Project ID </en/docs/acc/v1/tutorials/getting-started/retrieve-account-and-project-id/>`_ tutorial.You may provide the project ID with or without the ``b.`` prefix:- With prefix: ``b.657a5565-09b7-48e0-bd03-acacfe42efaf``- Without prefix: ``657a5565-09b7-48e0-bd03-acacfe42efaf``
        Returns: WithProjectItemRequestBuilder
        """
        if project_id is None:
            raise TypeError("project_id cannot be null.")
        from .item.with_project_item_request_builder import WithProjectItemRequestBuilder

        url_tpl_params = get_path_parameters(self.path_parameters)
        url_tpl_params["projectId"] = project_id
        return WithProjectItemRequestBuilder(self.request_adapter, url_tpl_params)
    




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union

if TYPE_CHECKING:
    from .assigned_post_response_clash_data_clashes import AssignedPostResponse_clashData_clashes
    from .assigned_post_response_clash_data_clash_instances import AssignedPostResponse_clashData_clashInstances
    from .assigned_post_response_clash_data_documents import AssignedPostResponse_clashData_documents

@dataclass
class AssignedPostResponse_clashData(Parsable):
    # The clash instances associated with the clash groups supplied.
    clash_instances: Optional[list[AssignedPostResponse_clashData_clashInstances]] = None
    # The clashes associated with the clash groups supplied.
    clashes: Optional[list[AssignedPostResponse_clashData_clashes]] = None
    # The documents associated with the clash groups supplied.
    documents: Optional[list[AssignedPostResponse_clashData_documents]] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> AssignedPostResponse_clashData:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: AssignedPostResponse_clashData
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return AssignedPostResponse_clashData()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        from .assigned_post_response_clash_data_clashes import AssignedPostResponse_clashData_clashes
        from .assigned_post_response_clash_data_clash_instances import AssignedPostResponse_clashData_clashInstances
        from .assigned_post_response_clash_data_documents import AssignedPostResponse_clashData_documents

        from .assigned_post_response_clash_data_clashes import AssignedPostResponse_clashData_clashes
        from .assigned_post_response_clash_data_clash_instances import AssignedPostResponse_clashData_clashInstances
        from .assigned_post_response_clash_data_documents import AssignedPostResponse_clashData_documents

        fields: dict[str, Callable[[Any], None]] = {
            "clashInstances": lambda n : setattr(self, 'clash_instances', n.get_collection_of_object_values(AssignedPostResponse_clashData_clashInstances)),
            "clashes": lambda n : setattr(self, 'clashes', n.get_collection_of_object_values(AssignedPostResponse_clashData_clashes)),
            "documents": lambda n : setattr(self, 'documents', n.get_collection_of_object_values(AssignedPostResponse_clashData_documents)),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_collection_of_object_values("clashInstances", self.clash_instances)
        writer.write_collection_of_object_values("clashes", self.clashes)
        writer.write_collection_of_object_values("documents", self.documents)
    




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.base_request_builder import BaseRequestBuilder
from kiota_abstractions.base_request_configuration import RequestConfiguration
from kiota_abstractions.default_query_parameters import QueryParameters
from kiota_abstractions.get_path_parameters import get_path_parameters
from kiota_abstractions.method import Method
from kiota_abstractions.request_adapter import RequestAdapter
from kiota_abstractions.request_information import RequestInformation
from kiota_abstractions.request_option import RequestOption
from kiota_abstractions.serialization import Parsable, ParsableFactory
from typing import Any, Optional, TYPE_CHECKING, Union
from uuid import UUID
from warnings import warn

if TYPE_CHECKING:
    from .item.item_request_builder import ItemRequestBuilder
    from .main_contracts_get_response import MainContractsGetResponse
    from .main_contracts_post_request_body import MainContractsPostRequestBody
    from .main_contracts_post_response import MainContractsPostResponse

class MainContractsRequestBuilder(BaseRequestBuilder):
    """
    Builds and executes requests for operations under /cost/v1/containers/{containerId}/main-contracts
    """
    def __init__(self,request_adapter: RequestAdapter, path_parameters: Union[str, dict[str, Any]]) -> None:
        """
        Instantiates a new MainContractsRequestBuilder and sets the default values.
        param path_parameters: The raw url or the url-template parameters for the request.
        param request_adapter: The request adapter to use to execute the requests.
        Returns: None
        """
        super().__init__(request_adapter, "{+baseurl}/cost/v1/containers/{containerId}/main-contracts{?filter%5Bcode%5D*,filter%5BexternalId%5D*,filter%5BexternalSystem%5D*,filter%5Bid%5D*,filter%5BlastModifiedSince%5D*,filter%5Bstatus%5D*,include*,limit*,offset*,sort*}", path_parameters)
    
    def by_id(self,id: UUID) -> ItemRequestBuilder:
        """
        Gets an item from the Autodesk.ACC.cost.v1.containers.item.mainContracts.item collection
        param id: The object ID of the main contract. You can obtain this ID from the response to the `POST mainContract </en/docs/bim360/v1/reference/http/cost-main-contracts-POST/>`_ or `GET mainContract </en/docs/bim360/v1/reference/http/cost-main-contracts-GET/>`_ endpoint.
        Returns: ItemRequestBuilder
        """
        if id is None:
            raise TypeError("id cannot be null.")
        from .item.item_request_builder import ItemRequestBuilder

        url_tpl_params = get_path_parameters(self.path_parameters)
        url_tpl_params["%2Did"] = id
        return ItemRequestBuilder(self.request_adapter, url_tpl_params)
    
    async def get(self,request_configuration: Optional[RequestConfiguration[MainContractsRequestBuilderGetQueryParameters]] = None) -> Optional[MainContractsGetResponse]:
        """
        Retrieves one or more of the main contracts in the given project.
        param request_configuration: Configuration for the request such as headers, query parameters, and middleware options.
        Returns: Optional[MainContractsGetResponse]
        """
        request_info = self.to_get_request_information(
            request_configuration
        )
        if not self.request_adapter:
            raise Exception("Http core is null") 
        from .main_contracts_get_response import MainContractsGetResponse

        return await self.request_adapter.send_async(request_info, MainContractsGetResponse, None)
    
    async def post(self,body: MainContractsPostRequestBody, request_configuration: Optional[RequestConfiguration[QueryParameters]] = None) -> Optional[MainContractsPostResponse]:
        """
        Creates a main contract in the given project.
        param body: The request body
        param request_configuration: Configuration for the request such as headers, query parameters, and middleware options.
        Returns: Optional[MainContractsPostResponse]
        """
        if body is None:
            raise TypeError("body cannot be null.")
        request_info = self.to_post_request_information(
            body, request_configuration
        )
        if not self.request_adapter:
            raise Exception("Http core is null") 
        from .main_contracts_post_response import MainContractsPostResponse

        return await self.request_adapter.send_async(request_info, MainContractsPostResponse, None)
    
    def to_get_request_information(self,request_configuration: Optional[RequestConfiguration[MainContractsRequestBuilderGetQueryParameters]] = None) -> RequestInformation:
        """
        Retrieves one or more of the main contracts in the given project.
        param request_configuration: Configuration for the request such as headers, query parameters, and middleware options.
        Returns: RequestInformation
        """
        request_info = RequestInformation(Method.GET, self.url_template, self.path_parameters)
        request_info.configure(request_configuration)
        request_info.headers.try_add("Accept", "application/json")
        return request_info
    
    def to_post_request_information(self,body: MainContractsPostRequestBody, request_configuration: Optional[RequestConfiguration[QueryParameters]] = None) -> RequestInformation:
        """
        Creates a main contract in the given project.
        param body: The request body
        param request_configuration: Configuration for the request such as headers, query parameters, and middleware options.
        Returns: RequestInformation
        """
        if body is None:
            raise TypeError("body cannot be null.")
        request_info = RequestInformation(Method.POST, self.url_template, self.path_parameters)
        request_info.configure(request_configuration)
        request_info.headers.try_add("Accept", "application/json")
        request_info.set_content_from_parsable(self.request_adapter, "application/json", body)
        return request_info
    
    def with_url(self,raw_url: str) -> MainContractsRequestBuilder:
        """
        Returns a request builder with the provided arbitrary URL. Using this method means any other path or query parameters are ignored.
        param raw_url: The raw URL to use for the request builder.
        Returns: MainContractsRequestBuilder
        """
        if raw_url is None:
            raise TypeError("raw_url cannot be null.")
        return MainContractsRequestBuilder(self.request_adapter, raw_url)
    
    @dataclass
    class MainContractsRequestBuilderGetQueryParameters():
        """
        Retrieves one or more of the main contracts in the given project.
        """
        def get_query_parameter(self,original_name: str) -> str:
            """
            Maps the query parameters names to their encoded names for the URI template parsing.
            param original_name: The original query parameter name in the class.
            Returns: str
            """
            if original_name is None:
                raise TypeError("original_name cannot be null.")
            if original_name == "filtercode":
                return "filter%5Bcode%5D"
            if original_name == "filterexternal_id":
                return "filter%5BexternalId%5D"
            if original_name == "filterexternal_system":
                return "filter%5BexternalSystem%5D"
            if original_name == "filterid":
                return "filter%5Bid%5D"
            if original_name == "filterlast_modified_since":
                return "filter%5BlastModifiedSince%5D"
            if original_name == "filterstatus":
                return "filter%5Bstatus%5D"
            if original_name == "include":
                return "include"
            if original_name == "limit":
                return "limit"
            if original_name == "offset":
                return "offset"
            if original_name == "sort":
                return "sort"
            return original_name
        
        # Returns only items that are identified by the specified codes (separated by commas). For example, ``filter[code]=``2703,2704````.Note that even single codes in number format must be enclosed in double quotes:``filter[code]=``1```` or ``filter[code]=``122. 221````.
        filtercode: Optional[list[str]] = None

        # The ID of the item in the external ERP system. Use this ID to track or look up data in an integrated ERP system. For example, ``filter[externalId]=id1,id2``.
        filterexternal_id: Optional[list[str]] = None

        # The name of the external ERP system. Use this name to identify or search within the integrated system. For example, ``filter[externalSystem]=Sage300``.Max length: 255
        filterexternal_system: Optional[str] = None

        # Returns only the items that are identified by the provided list of item IDs. Separate multiple IDs with commas. For example, ``filter[id]=id1,id2``.
        filterid: Optional[list[str]] = None

        # Returns only items that were modified since the specified date and time, in ISO 8601 format. For example, ``filter[lastModifiedSince]=2020-03-01T13:00:00Z``.
        filterlast_modified_since: Optional[str] = None

        # Returns only main contracts that have a status on the provided list of statuses. Separate multiple values with commas; for example, ``filter[status]=draft,pending``.Possible values:``draft``, ``pending``, ``submitted``, ``revise``, ``sent``, ``signed``, ``executed``, ``closed``, ``inReview``.
        filterstatus: Optional[list[str]] = None

        # A list of the types of sub-main contract components to include in the response with the main contracts, separated by commas.For example, ``include=mainContractItems`` returns the items that comprise each of the returned main contracts; ``include=attributes`` returns custom attributes of the main contracts.Possible values ``mainContractItems``, ``attributes``.
        include: Optional[list[str]] = None

        # The maximum number of records returned per page. Default: ``100``. A page may contain fewer records than the limit if there are fewer matching items or if it is the last page of results.
        limit: Optional[int] = None

        # The number of records to skip before returning results. Used together with ``limit`` to paginate through results, where ``offset`` specifies the starting point and ``limit`` specifies the number of records to return.
        offset: Optional[int] = None

        # Defines the sort order for the results. Each attribute can be sorted in ``asc`` (default) or ``desc`` order. For example, ``sort=name desc`` sorts the results by name in descending order.
        sort: Optional[str] = None

    
    @dataclass
    class MainContractsRequestBuilderGetRequestConfiguration(RequestConfiguration[MainContractsRequestBuilderGetQueryParameters]):
        """
        Configuration for the request such as headers, query parameters, and middleware options.
        """
        warn("This class is deprecated. Please use the generic RequestConfiguration class generated by the generator.", DeprecationWarning)
    
    @dataclass
    class MainContractsRequestBuilderPostRequestConfiguration(RequestConfiguration[QueryParameters]):
        """
        Configuration for the request such as headers, query parameters, and middleware options.
        """
        warn("This class is deprecated. Please use the generic RequestConfiguration class generated by the generator.", DeprecationWarning)
    




from __future__ import annotations
import datetime
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union

if TYPE_CHECKING:
    from .documents_get_response_results_approve_status import DocumentsGetResponse_results_approveStatus

@dataclass
class DocumentsGetResponse_results(Parsable):
    # The approval status of the file version.For more information, see `File Review Statuses <https://help.autodesk.com/view/DOCS/ENU/?guid=BIM360D_Document_Management_About_Reviews_Reviews_FAQs_Reference_html#file-review-statuses>`_ documentation.
    approve_status: Optional[DocumentsGetResponse_results_approveStatus] = None
    # The description of the document, if provided.
    description: Optional[str] = None
    # The file name of the document.
    file_name: Optional[str] = None
    # The type of folder that contains the document.
    folder_type: Optional[str] = None
    # Indicates whether the file is deleted.``true`` – The file is deleted, either directly or because its parent folder was deleted.``false`` – The file is not deleted.
    is_deleted: Optional[bool] = None
    # The date and time when the file was last updated, in ISO 8601 format.
    last_updated_at: Optional[datetime.datetime] = None
    # The name of the document.
    name: Optional[str] = None
    # The URN of the folder that contains the document.
    parent_folder_urn: Optional[str] = None
    # The revision label assigned to the document.
    revision_label: Optional[str] = None
    # The storage URN of the document. You can use the storage URN to download the document. For details, see the `Download Files </en/docs/acc/v1/tutorials/files/download-document-s3/>`_ tutorial.
    storage_urn: Optional[str] = None
    # The title of the document.
    title: Optional[str] = None
    # The Autodesk ID of the user who created the file. For details about the user, call `GET user </en/docs/acc/v1/reference/http/admin-projectsprojectId-users-userId-GET/>`_.
    updated_by: Optional[str] = None
    # The name of the user who last modified the document.
    updated_by_name: Optional[str] = None
    # The URN of the document.
    urn: Optional[str] = None
    # The version number of the document at the time the transmittal was issued.
    version: Optional[int] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> DocumentsGetResponse_results:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: DocumentsGetResponse_results
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return DocumentsGetResponse_results()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        from .documents_get_response_results_approve_status import DocumentsGetResponse_results_approveStatus

        from .documents_get_response_results_approve_status import DocumentsGetResponse_results_approveStatus

        fields: dict[str, Callable[[Any], None]] = {
            "approveStatus": lambda n : setattr(self, 'approve_status', n.get_object_value(DocumentsGetResponse_results_approveStatus)),
            "description": lambda n : setattr(self, 'description', n.get_str_value()),
            "fileName": lambda n : setattr(self, 'file_name', n.get_str_value()),
            "folderType": lambda n : setattr(self, 'folder_type', n.get_str_value()),
            "isDeleted": lambda n : setattr(self, 'is_deleted', n.get_bool_value()),
            "lastUpdatedAt": lambda n : setattr(self, 'last_updated_at', n.get_datetime_value()),
            "name": lambda n : setattr(self, 'name', n.get_str_value()),
            "parentFolderUrn": lambda n : setattr(self, 'parent_folder_urn', n.get_str_value()),
            "revisionLabel": lambda n : setattr(self, 'revision_label', n.get_str_value()),
            "storageUrn": lambda n : setattr(self, 'storage_urn', n.get_str_value()),
            "title": lambda n : setattr(self, 'title', n.get_str_value()),
            "updatedBy": lambda n : setattr(self, 'updated_by', n.get_str_value()),
            "updatedByName": lambda n : setattr(self, 'updated_by_name', n.get_str_value()),
            "urn": lambda n : setattr(self, 'urn', n.get_str_value()),
            "version": lambda n : setattr(self, 'version', n.get_int_value()),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_object_value("approveStatus", self.approve_status)
        writer.write_str_value("description", self.description)
        writer.write_str_value("fileName", self.file_name)
        writer.write_str_value("folderType", self.folder_type)
        writer.write_bool_value("isDeleted", self.is_deleted)
        writer.write_datetime_value("lastUpdatedAt", self.last_updated_at)
        writer.write_str_value("name", self.name)
        writer.write_str_value("parentFolderUrn", self.parent_folder_urn)
        writer.write_str_value("revisionLabel", self.revision_label)
        writer.write_str_value("storageUrn", self.storage_urn)
        writer.write_str_value("title", self.title)
        writer.write_str_value("updatedBy", self.updated_by)
        writer.write_str_value("updatedByName", self.updated_by_name)
        writer.write_str_value("urn", self.urn)
        writer.write_int_value("version", self.version)
    




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union

if TYPE_CHECKING:
    from .sheets_batch_update_post_request_body_updates import SheetsBatchUpdatePostRequestBody_updates

@dataclass
class SheetsBatchUpdatePostRequestBody(Parsable):
    # To find the IDs of the sheets you want to update, call `GET sheets </en/docs/acc/v1/reference/http/sheets-sheets-GET/>`_.- The max number of items is 200.
    ids: Optional[list[str]] = None
    # The list of updates.
    updates: Optional[list[SheetsBatchUpdatePostRequestBody_updates]] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> SheetsBatchUpdatePostRequestBody:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: SheetsBatchUpdatePostRequestBody
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return SheetsBatchUpdatePostRequestBody()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        from .sheets_batch_update_post_request_body_updates import SheetsBatchUpdatePostRequestBody_updates

        from .sheets_batch_update_post_request_body_updates import SheetsBatchUpdatePostRequestBody_updates

        fields: dict[str, Callable[[Any], None]] = {
            "ids": lambda n : setattr(self, 'ids', n.get_collection_of_primitive_values(str)),
            "updates": lambda n : setattr(self, 'updates', n.get_collection_of_object_values(SheetsBatchUpdatePostRequestBody_updates)),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_collection_of_primitive_values("ids", self.ids)
        writer.write_collection_of_object_values("updates", self.updates)
    




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union

@dataclass
class TaxesGetResponse_results_summary(Parsable):
    # The total amount of the applied tax.
    amount: Optional[float] = None
    # The name of the applied tax (e.g., Sales Tax, VAT, State Tax, City Tax, Local Tax).
    name: Optional[str] = None
    # (optional) The rate of the tax applied.
    rate: Optional[str] = None
    # The type of tax applied, such as ``rate``.
    type: Optional[str] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> TaxesGetResponse_results_summary:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: TaxesGetResponse_results_summary
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return TaxesGetResponse_results_summary()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        fields: dict[str, Callable[[Any], None]] = {
            "amount": lambda n : setattr(self, 'amount', n.get_float_value()),
            "name": lambda n : setattr(self, 'name', n.get_str_value()),
            "rate": lambda n : setattr(self, 'rate', n.get_str_value()),
            "type": lambda n : setattr(self, 'type', n.get_str_value()),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_float_value("amount", self.amount)
        writer.write_str_value("name", self.name)
        writer.write_str_value("rate", self.rate)
        writer.write_str_value("type", self.type)
    




from __future__ import annotations
import datetime
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.base_request_builder import BaseRequestBuilder
from kiota_abstractions.base_request_configuration import RequestConfiguration
from kiota_abstractions.default_query_parameters import QueryParameters
from kiota_abstractions.get_path_parameters import get_path_parameters
from kiota_abstractions.method import Method
from kiota_abstractions.request_adapter import RequestAdapter
from kiota_abstractions.request_information import RequestInformation
from kiota_abstractions.request_option import RequestOption
from kiota_abstractions.serialization import Parsable, ParsableFactory
from typing import Any, Optional, TYPE_CHECKING, Union
from warnings import warn

if TYPE_CHECKING:
    from .forms_get_response import FormsGetResponse
    from .item.with_form_item_request_builder import WithFormItemRequestBuilder

class FormsRequestBuilder(BaseRequestBuilder):
    """
    Builds and executes requests for operations under /construction/forms/v1/projects/{projectId}/forms
    """
    def __init__(self,request_adapter: RequestAdapter, path_parameters: Union[str, dict[str, Any]]) -> None:
        """
        Instantiates a new FormsRequestBuilder and sets the default values.
        param path_parameters: The raw url or the url-template parameters for the request.
        param request_adapter: The request adapter to use to execute the requests.
        Returns: None
        """
        super().__init__(request_adapter, "{+baseurl}/construction/forms/v1/projects/{projectId}/forms{?formDateMax*,formDateMin*,ids*,limit*,locationIds*,offset*,sortBy*,sortOrder*,statuses*,templateId*,updatedAfter*,updatedBefore*}", path_parameters)
    
    def by_form_id(self,form_id: str) -> WithFormItemRequestBuilder:
        """
        Gets an item from the Autodesk.ACC.construction.forms.v1.projects.item.forms.item collection
        param form_id: The unique identifier of the form.Use `GET forms </en/docs/acc/v1/reference/http/forms-forms-GET/>`_ to retrieve the form ID.
        Returns: WithFormItemRequestBuilder
        """
        if form_id is None:
            raise TypeError("form_id cannot be null.")
        from .item.with_form_item_request_builder import WithFormItemRequestBuilder

        url_tpl_params = get_path_parameters(self.path_parameters)
        url_tpl_params["formId"] = form_id
        return WithFormItemRequestBuilder(self.request_adapter, url_tpl_params)
    
    async def get(self,request_configuration: Optional[RequestConfiguration[FormsRequestBuilderGetQueryParameters]] = None) -> Optional[FormsGetResponse]:
        """
        Returns a paginated list of forms in a project. Forms are sorted by updatedAt, most recent first.
        param request_configuration: Configuration for the request such as headers, query parameters, and middleware options.
        Returns: Optional[FormsGetResponse]
        """
        request_info = self.to_get_request_information(
            request_configuration
        )
        if not self.request_adapter:
            raise Exception("Http core is null") 
        from .forms_get_response import FormsGetResponse

        return await self.request_adapter.send_async(request_info, FormsGetResponse, None)
    
    def to_get_request_information(self,request_configuration: Optional[RequestConfiguration[FormsRequestBuilderGetQueryParameters]] = None) -> RequestInformation:
        """
        Returns a paginated list of forms in a project. Forms are sorted by updatedAt, most recent first.
        param request_configuration: Configuration for the request such as headers, query parameters, and middleware options.
        Returns: RequestInformation
        """
        request_info = RequestInformation(Method.GET, self.url_template, self.path_parameters)
        request_info.configure(request_configuration)
        request_info.headers.try_add("Accept", "application/json")
        return request_info
    
    def with_url(self,raw_url: str) -> FormsRequestBuilder:
        """
        Returns a request builder with the provided arbitrary URL. Using this method means any other path or query parameters are ignored.
        param raw_url: The raw URL to use for the request builder.
        Returns: FormsRequestBuilder
        """
        if raw_url is None:
            raise TypeError("raw_url cannot be null.")
        return FormsRequestBuilder(self.request_adapter, raw_url)
    
    @dataclass
    class FormsRequestBuilderGetQueryParameters():
        """
        Returns a paginated list of forms in a project. Forms are sorted by updatedAt, most recent first.
        """
        def get_query_parameter(self,original_name: str) -> str:
            """
            Maps the query parameters names to their encoded names for the URI template parsing.
            param original_name: The original query parameter name in the class.
            Returns: str
            """
            if original_name is None:
                raise TypeError("original_name cannot be null.")
            if original_name == "form_date_max":
                return "formDateMax"
            if original_name == "form_date_min":
                return "formDateMin"
            if original_name == "location_ids":
                return "locationIds"
            if original_name == "sort_by":
                return "sortBy"
            if original_name == "sort_order":
                return "sortOrder"
            if original_name == "template_id":
                return "templateId"
            if original_name == "updated_after":
                return "updatedAfter"
            if original_name == "updated_before":
                return "updatedBefore"
            if original_name == "ids":
                return "ids"
            if original_name == "limit":
                return "limit"
            if original_name == "offset":
                return "offset"
            if original_name == "statuses":
                return "statuses"
            return original_name
        
        # Return Forms with formDate at or before specified date.
        form_date_max: Optional[str] = None

        # Return Forms with formDate at or after specified date.
        form_date_min: Optional[str] = None

        # An array of Form IDs to retrieve.
        ids: Optional[list[str]] = None

        # The number of records to return in a single request. Can be a number between 1 and 50. Defaults to 50.
        limit: Optional[int] = None

        # A sequence of location IDs. Each returned object must be associated with one of the locations specified by the IDs. For example, ?locationId=123e102a-36de-14e7-8c56-1b1234ccbba8&locationId=cee45678-fcc4-43ae-80a2-8ca819dfa70d. See the usage example in the `Retrieve Forms Associated With Locations </en/docs/acc/v1/tutorials/forms/retrieve-forms-based-on-locations/>`_ tutorial.
        location_ids: Optional[list[str]] = None

        # The number of records to skip before returning the result records. Defaults to 0. Increase this value in subsequent requests to continue getting results when the number of records exceeds the requested limit.
        offset: Optional[int] = None

        # Return Forms sorted by specified attribute.
        sort_by: Optional[str] = None

        # Return Forms in specified sorted order.
        sort_order: Optional[str] = None

        # Return Forms with given statuses.
        statuses: Optional[list[str]] = None

        # Return Forms on template with given ID.
        template_id: Optional[str] = None

        # Return Forms updated after a specified time.
        updated_after: Optional[datetime.datetime] = None

        # Return Forms updated before a specified time.
        updated_before: Optional[datetime.datetime] = None

    
    @dataclass
    class FormsRequestBuilderGetRequestConfiguration(RequestConfiguration[FormsRequestBuilderGetQueryParameters]):
        """
        Configuration for the request such as headers, query parameters, and middleware options.
        """
        warn("This class is deprecated. Please use the generic RequestConfiguration class generated by the generator.", DeprecationWarning)
    




from enum import Enum

class PropertiesGetResponse_type(str, Enum):
    Text = "text",
    Multiline = "multiline",
    Richtext = "richtext",
    Options = "options",
    Boolean = "boolean",
    Integer = "integer",
    Percent = "percent",
    Currency = "currency",
    Number = "number",
    Datetime = "datetime",




from __future__ import annotations
from collections.abc import Callable
from dataclasses import dataclass, field
from kiota_abstractions.serialization import Parsable, ParseNode, SerializationWriter
from typing import Any, Optional, TYPE_CHECKING, Union

if TYPE_CHECKING:
    from .assets_get_response_results_custom_attributes_asterisk import AssetsGetResponse_results_customAttributes_Asterisk

@dataclass
class AssetsGetResponse_results_customAttributes(Parsable):
    """
    An optional JSON dictionary specifying one or more custom attributes and values to be assigned to theasset. The custom attributes must belong to the set of custom attributes specified by the asset's category.The dictionary is a set of key:value pairs that each starts with the unique custom attribute name (not tobe confused with the custom attribute's display name) followed by the attribute value. The value must usethe data type defined by the attribute:For ``text``, the value is a string.For ``date``, the value is an ISO8601 date string with no time, for example, ``2020-04-10``.For ``select``, the value is a valid ID from the list of values defined for this custom attribute.For ``multi-select``, the value is an array of valid IDs from the list of values defined for this custom attribute.For ``boolean``, the value is a boolean.For ``numeric``, the value is a string that parses as a valid floating point number (not localized).
    """
    # The Asterisk property
    asterisk: Optional[AssetsGetResponse_results_customAttributes_Asterisk] = None
    
    @staticmethod
    def create_from_discriminator_value(parse_node: ParseNode) -> AssetsGetResponse_results_customAttributes:
        """
        Creates a new instance of the appropriate class based on discriminator value
        param parse_node: The parse node to use to read the discriminator value and create the object
        Returns: AssetsGetResponse_results_customAttributes
        """
        if parse_node is None:
            raise TypeError("parse_node cannot be null.")
        return AssetsGetResponse_results_customAttributes()
    
    def get_field_deserializers(self,) -> dict[str, Callable[[ParseNode], None]]:
        """
        The deserialization information for the current model
        Returns: dict[str, Callable[[ParseNode], None]]
        """
        from .assets_get_response_results_custom_attributes_asterisk import AssetsGetResponse_results_customAttributes_Asterisk

        from .assets_get_response_results_custom_attributes_asterisk import AssetsGetResponse_results_customAttributes_Asterisk

        fields: dict[str, Callable[[Any], None]] = {
            "*": lambda n : setattr(self, 'asterisk', n.get_object_value(AssetsGetResponse_results_customAttributes_Asterisk)),
        }
        return fields
    
    def serialize(self,writer: SerializationWriter) -> None:
        """
        Serializes information the current object
        param writer: Serialization writer to use to serialize this model
        Returns: None
        """
        if writer is None:
            raise TypeError("writer cannot be null.")
        writer.write_object_value("*", self.asterisk)
    




from enum import Enum

class WithRfiPatchResponse_permittedActions_updateRfi_permittedStatuses_wfUS_status(str, Enum):
    Draft = "draft",
    Submitted = "submitted",
    Open = "open",
    Answered = "answered",
    Rejected = "rejected",
    Closed = "closed",
    Void = "void",




from __future__ import annotations
from collections.abc import Callable
from kiota_abstractions.base_request_builder import BaseRequestBuilder
from kiota_abstractions.get_path_parameters import get_path_parameters
from kiota_abstractions.request_adapter import RequestAdapter
from typing import Any, Optional, TYPE_CHECKING, Union
from uuid import UUID

if TYPE_CHECKING:
    from .item.with_association_item_request_builder import WithAssociationItemRequestBuilder

class WithAssociationTypeItemRequestBuilder(BaseRequestBuilder):
    """
    Builds and executes requests for operations under /cost/v1/containers/{containerId}/workflows/{associationType}
    """
    def __init__(self,request_adapter: RequestAdapter, path_parameters: Union[str, dict[str, Any]]) -> None:
        """
        Instantiates a new WithAssociationTypeItemRequestBuilder and sets the default values.
        param path_parameters: The raw url or the url-template parameters for the request.
        param request_adapter: The request adapter to use to execute the requests.
        Returns: None
        """
        super().__init__(request_adapter, "{+baseurl}/cost/v1/containers/{containerId}/workflows/{associationType}", path_parameters)
    
    def by_association_id(self,association_id: UUID) -> WithAssociationItemRequestBuilder:
        """
        Gets an item from the Autodesk.ACC.cost.v1.containers.item.workflows.item.item collection
        param association_id: The object ID of the item is associated to. For example, the ID of a budget, contract or cost item.
        Returns: WithAssociationItemRequestBuilder
        """
        if association_id is None:
            raise TypeError("association_id cannot be null.")
        from .item.with_association_item_request_builder import WithAssociationItemRequestBuilder

        url_tpl_params = get_path_parameters(self.path_parameters)
        url_tpl_params["associationId"] = association_id
        return WithAssociationItemRequestBuilder(self.request_adapter, url_tpl_params)
    




from enum import Enum

class WithProjectGetResponse_projectValue_currency(str, Enum):
    USD = "USD",
    AED = "AED",
    AFN = "AFN",
    ALL = "ALL",
    AMD = "AMD",
    ANG = "ANG",
    AOA = "AOA",
    ARS = "ARS",
    AUD = "AUD",
    AWG = "AWG",
    AZN = "AZN",
    BAM = "BAM",
    BBD = "BBD",
    BDT = "BDT",
    BGN = "BGN",
    BHD = "BHD",
    BIF = "BIF",
    BMD = "BMD",
    BND = "BND",
    BOB = "BOB",
    BOV = "BOV",
    BRL = "BRL",
    BSD = "BSD",
    BTN = "BTN",
    BWP = "BWP",
    BYN = "BYN",
    BYR = "BYR",
    BZD = "BZD",
    CAD = "CAD",
    CDF = "CDF",
    CHE = "CHE",
    CHF = "CHF",
    CHW = "CHW",
    CLF = "CLF",
    CLP = "CLP",
    CNY = "CNY",
    COP = "COP",
    COU = "COU",
    CRC = "CRC",
    CUC = "CUC",
    CUP = "CUP",
    CVE = "CVE",
    CZK = "CZK",
    DJF = "DJF",
    DKK = "DKK",
    DOP = "DOP",
    DZD = "DZD",
    EEK = "EEK",
    EGP = "EGP",
    ERN = "ERN",
    ETB = "ETB",
    EUR = "EUR",
    FJD = "FJD",
    FKP = "FKP",
    GBP = "GBP",
    GEL = "GEL",
    GHS = "GHS",
    GIP = "GIP",
    GMD = "GMD",
    GNF = "GNF",
    GTQ = "GTQ",
    GYD = "GYD",
    HKD = "HKD",
    HNL = "HNL",
    HRK = "HRK",
    HTG = "HTG",
    HUF = "HUF",
    IDR = "IDR",
    ILS = "ILS",
    INR = "INR",
    IQD = "IQD",
    IRR = "IRR",
    ISK = "ISK",
    JMD = "JMD",
    JOD = "JOD",
    JPY = "JPY",
    KES = "KES",
    KGS = "KGS",
    KHR = "KHR",
    KMF = "KMF",
    KPW = "KPW",
    KRW = "KRW",
    KWD = "KWD",
    KYD = "KYD",
    KZT = "KZT",
    LAK = "LAK",
    LBP = "LBP",
    LKR = "LKR",
    LRD = "LRD",
    LSL = "LSL",
    LTL = "LTL",
    LVL = "LVL",
    LYD = "LYD",
    MAD = "MAD",
    MDL = "MDL",
    MGA = "MGA",
    MKD = "MKD",
    MMK = "MMK",
    MNT = "MNT",
    MOP = "MOP",
    MRU = "MRU",
    MUR = "MUR",
    MVR = "MVR",
    MWK = "MWK",
    MXN = "MXN",
    MXV = "MXV",
    MYR = "MYR",
    MZN = "MZN",
    NAD = "NAD",
    NGN = "NGN",
    NIO = "NIO",
    NOK = "NOK",
    NPR = "NPR",
    NZD = "NZD",
    OMR = "OMR",
    PAB = "PAB",
    PEN = "PEN",
    PGK = "PGK",
    PHP = "PHP",
    PKR = "PKR",
    PLN = "PLN",
    PYG = "PYG",
    QAR = "QAR",
    RON = "RON",
    RSD = "RSD",
    RUB = "RUB",
    RWF = "RWF",
    SAR = "SAR",
    SBD = "SBD",
    SCR = "SCR",
    SDG = "SDG",
    SEK = "SEK",
    SGD = "SGD",
    SHP = "SHP",
    SLE = "SLE",
    SLL = "SLL",
    SOS = "SOS",
    SRD = "SRD",
    SSP = "SSP",
    STN = "STN",
    SVC = "SVC",
    SYP = "SYP",
    SZL = "SZL",
    THB = "THB",
    TJS = "TJS",
    TMT = "TMT",
    TND = "TND",
    TOP = "TOP",
    TRL = "TRL",
    TRY_ = "TRY",
    TTD = "TTD",
    TWD = "TWD",
    TZS = "TZS",
    UAH = "UAH",
    UGX = "UGX",
    USN = "USN",
    UYI = "UYI",
    UYU = "UYU",
    UYW = "UYW",
    UZS = "UZS",
    VED = "VED",
    VES = "VES",
    VND = "VND",
    VUV = "VUV",
    WST = "WST",
    XAF = "XAF",
    XAG = "XAG",
    XAU = "XAU",
    XBA = "XBA",
    XBB = "XBB",
    XBC = "XBC",
    XBD = "XBD",
    XCD = "XCD",
    XDR = "XDR",
    XOF = "XOF",
    XPD = "XPD",
    XPF = "XPF",
    XPT = "XPT",
    XSU = "XSU",
    XTS = "XTS",
    XUA = "XUA",
    XXX = "XXX",
    YER = "YER",
    ZAR = "ZAR",
    ZMW = "ZMW",
    ZWL = "ZWL",

